/******************************************************************************
 * contains command tables for the editors of the eshell
 *****************************************************************************/

static editCmd viewCmdTable [] = {
  {ctrl('a'), beglin},		/* beginning of line */
  {ctrl('b'), left},		/* cursor left  */
  {ctrl('c'), noop},		/* illegal */
  {ctrl('d'), noop},		/* illegal */
  {ctrl('e'), endlin},		/* end of line */
  {ctrl('f'), right},		/* cursor right */
  {ctrl('g'), noop},		/* illegal */
  {ctrl('h'), noop},		/* illegal */
  {ctrl('i'), noop},		/* illegal */
  {ctrl('j'), noop},		/* illegal */
  {ctrl('k'), noop},		/* illegal */
  {ctrl('l'), newscr},		/* redraw screen */
  {ctrl('m'), noop},		/* illegal */
  {ctrl('n'), dwncol},		/* down in column */
  {ctrl('o'), noop},		/* illegal */
  {ctrl('p'), upcol},		/* up in column */
  {ctrl('q'), noop},		/* illegal */
  {ctrl('r'), bwsearch},	/* search backward */
  {ctrl('s'), fsearch},		/* search forward */
  {ctrl('t'), noop},		/* illegal */
  {ctrl('u'), noop},		/* illegal */
  {ctrl('v'), uppage},		/* forward page */
  {ctrl('w'), noop},		/* illegal */
  {ctrl('x'), noop},		/* illegal */
  {ctrl('y'), noop},		/* illegal */
  {ctrl('z'), noop},		/* illegal */
  {EOS,     (int (*)()) 0},	/* end of list */
};		

static editCmd editCmdTable [] = {
  {ctrl('a'), beglin},		/* beginning of line */
  {ctrl('b'), left},		/* cursor left  */
  {ctrl('c'), intr},		/* send a interupt-signal */
  {ctrl('d'), delnxt},	 	/* delete next character */
  {ctrl('e'), endlin},		/* end of line */
  {ctrl('f'), right},		/* cursor right */
  {ctrl('g'), noop},		/* illegal */
  {ctrl('h'), noop},		/* illegal */
  {ctrl('i'), completeName},	/* complete a file name */
  {ctrl('j'), sendLine},	/* send a line to the application */
  {ctrl('k'), krest},		/* delete to end of line */
  {ctrl('l'), newscr},		/* redraw screen */
  {ctrl('m'), sendLine},	/* send a line to the application */
  {ctrl('n'), dwncol},		/* down in column */
  {ctrl('o'), openln},		/* open line above */
  {ctrl('p'), upcol},		/* up in column */
  {ctrl('q'), noop},		/* illegal */
  {ctrl('r'), bwsearch},	/* search backward */
  {ctrl('s'), fsearch},		/* search forward */
  {ctrl('t'), noop},		/* illegal */
  {ctrl('u'), showCompletions},	/* show the possible completions */
  {ctrl('v'), uppage},		/* forward page */
  {ctrl('w'), noop},		/* illegal */
  {ctrl('x'), noop},		/* illegal */
  {ctrl('\\'), quit},		/* send a quit-signal */
  {ctrl('z'), stop},	     	/* send a stop-signal */
  {127,     delprev},		/* delete previous char */
  {EOS,     (int (*)()) 0},	/* end of list */
};		

static editCmd ctrlXCmds [] = {		       
  {ctrl('d'), send_eof},  	/* send an end of line character */
  {ctrl('f'), findFile},	/* read in alternate file */
  {ctrl('i'), insertFile},	/* insert named file into buffer */
  {ctrl('l'), lastCmd}, 	/* retrieve 'last' cmd from history tracer */
  {ctrl('n'), nextCmd}, 	/* retrieve 'next' cmd from history tracer */
  {ctrl('t'), thisCmd}, 	/* retrieve current cmd from history tracer */
  {ctrl('o'), openlnd},		/* open line below */
  {ctrl('p'), prevCmd}, 	/* get 'previous' cmd from history tracer */
  {ctrl('s'), saveFile},	/* save buffer to current file */
  {ctrl('w'), writeFile},	/* write buffer to named file */
  {'i', expand},                /* expand a command line */
  {'k', clearBuffer},		/* clear the current buffer */
  {'t', lineToTop},		/* current line to top of screen */
  {'?', help},			/* display help information */
  {EOS, (int (*)()) 0},		
};

static editCmd viewEscCmds [] = {
  {'<', topBuffer},		/* goto top of file */
  {'>', botBuffer},		/* goto bottom of file */
  {' ', goLine},		/* goto specified line */
  {'=', memory},		/* display statistics */
  {'b', wordl},			/* move to left word */
  {'f', wordr},			/* move to right word */
  {'h', topscr},		/* cursor to top of display */
  {'l', botscr},		/* cursor to last line of display */
  {'m', midscr},		/* cursor near middle of display */
  {'n', upScroll},		/* scroll up */
  {'p', dwnScroll},		/* scroll down */
  {'r', bbsearch},		/* continue search backword */
  {'s', ffsearch},		/* continue search forward */
  {'v', dwnpage},		/* display previous page */
  {EOS, (int (*)()) 0},		
};

static editCmd editEscCmds [] = {
  {'<', topBuffer},		/* goto top of file */
  {'>', botBuffer},		/* goto bottom of file */
  {' ', goLine},		/* goto specified line */
  {'=', memory},		/* display statistics */
  {'b', wordl},			/* move to left word */
  {'d', fdelword},		/* delete next word */
  {'f', wordr},			/* move to right word */
  {'h', topscr},		/* cursor to top of display */
  {'l', botscr},		/* cursor to last line of display */
  {'n', upScroll},		/* scroll down in cmd-history*/
  {'p', dwnScroll},		/* scroll up in cmd-history */
  {'r', bbsearch},		/* continue search backword */
  {'s', ffsearch},		/* continue search forward */
  {'v', dwnpage},		/* display previous page */
  {'w', fdelword},		/* delete next word */
  {ctrl('m'), handleMenu},	/* call a menu */
  {ctrl('p'), showPopups},      /* show the loaded popupup menus */
  {127, bdelword},		/* delete previous word */
  {EOS, (int (*)()) 0},		
};

static cmdDescr clCmds[] = {
  {sendLine, "  Request execution of the contents of the command editor."},
  {prevCmd,  "  Insert previous command from the history tracer."},
  {nextCmd,  "  Insert next command from the history tracer."},
  {thisCmd,  "  Insert current command from the history tracer."},
  {lastCmd,  "  Insert last command from the history tracer."},
  {expand,   "  Expand all file patterns in the current command line."},
  {send_eof, "  Quit the underlying UNIX-shell (i.e. send an end-of-line character (ctrl-d) to the shell)."},
  {intr, "  Interrupt a running foreground process started from the command editor (i.e. send an interrupt signal to the shell)."},
  {quit, "  Quit a running foreground process started from the command editor (i.e. send an quit signal to the shell)."},
  {stop, "  Stop a running foreground process started from the command editor (i.e. send a stop signal to the shell)."},
  {handleMenu, "  Take the current word as pop-up menu name and execute."},
  {completeName, "  Complete a partial file name."},
  {showCompletions, "  Show the possible completions for a partial file name."},
  {showPopups, "  Show the currently loaded pop-up menus."}
};

