/************************************************************************
 *                                                                      *
 *    Copyright (c) 1991 Wilhelm Valder, Regine Freitag, Wolfgang Dzida *
 *                        All rights reserved                           *
 *                                                                      *
 *                                                                      *
 *    Permission to copy this program or any part of this program is    *
 *    granted in case of personal, non-commercial use only. Any use     *
 *    for profit or other commercial gain without written permissions   *
 *    of the authors is prohibited.                                     *
 *                                                                      *
 *    You may freely distribute this program to friends, colleagues,    *
 *    etc.  As a courtesy we would appreciate it if you left our names  *
 *    on the code as the authors.                                       *
 *    We also, of course, disavow any liabilities                       *
 *    for any bizarre things you may choose to do with the system.      *
 *                                                                      *
 *    We would appreciate though your letting us know about any         *
 *    extensions, modifications, project suggestions, etc.              *
 *    Please mail all bugs, comments, complaints and suggestions to:    *
 *                                                                      *
 *                                                                      *
 *    GMD (German National Research Center for Computer Science)        *
 *    Institut fuer Systemtechnik (F2G2)                                *
 *    P.O. Box 1240                                                     *
 *    Schloss Birlinghoven                                              *
 *    D-5205 St. Augustin 1, West Germany                               *
 *                                                                      *
 *    email:    dzida@kmx.gmd.dbp.de                                    *
 *              freitag@gmdzi.uunet                                     *
 *                                                                      *
 ************************************************************************/
#include "fm.h"
#include "cmds.h"

#include <signal.h>
#include <sys/dir.h>

#include <icons/dirwIcon>
#include <pixels/closeBox>

#include <X11/Xutil.h>
#include <X11/cursorfont.h>

#include <Xm/Xm.h>
#include <Xm/DrawingA.h>
#include <Xm/Form.h>
#include <Xm/Label.h>
#include <Xm/PushB.h>

static char SccsID[] = "@(#)alloc_dir.c	1.4\t1/18/93";            /* sccs keyword */

static void createAttributeDisplay();
static forcedQuit();
extern fmQuit();

extern aedForm *exCreateAedForm();

void redrawIcon();
void closeDirWindow();
void openDirWindow();
void setTitle();
void fmRedraw();
void fmSetCursor();

static void updateIfNecessary();

extern buildTree(/* parms */);
extern cleanupDirWindows(/* parms */);
extern cpFile(/* parms */);
extern delFile(/* parms */);
extern commit(/* parms */);
extern editAttr(/* parms */);
extern fmAbort(/* parms */);
extern fmQuit(/* parms */);
extern listDirAny(/* parms */);
extern listDirFather(/* parms */);
extern listDirSel(/* parms */);
extern mvFile(/* parms */);
extern remDirWnd(/* parms */);
extern renFile(/* parms */);
extern selAll(/* parms */);
extern sortByAccess(/* parms */);
extern sortByMod(/* parms */);
extern sortByName(/* parms */);
extern sortBySize(/* parms */);
extern sortHorizontal(/* parms */);
extern sortVertical(/* parms */);
extern undelFile(/* parms */);
extern unrenFile(/* parms */);
extern unselAll(/* parms */);
extern dupFile(/* parms */);
extern selByGrep(/* parms */);
extern getFilePattern(/* parms */);
extern showFileAttributes(/* parms */);
extern putCurrentNames(/* parms */);
extern putAllNames(/* parms */);
extern makeDir(/* parms */);

dirWindow *allocateDirWindow();

static ApplicationNode *fm_aptr;

Cursor TreeCursor, CopyCursor, MoveCursor, PanelCursor;

Pixmap iconPixmap, closeBoxPixmap;

unsigned long fm_white_pixel, fm_black_pixel;

/* initFm: *******************************************************************
 * 
 *****************************************************************************/

initFM(parent)
     Widget parent;
{
  Display *display;
  extern char *getenv();
  char fullName[256];
  Window w;
  int cnt, depth;
  
  display = XtDisplay(parent);
  w = XDefaultRootWindow(display);

  exInitLayout(display);
  exInitMenus(display);

  initAed(display);
  
  signal(SIGINT,SIG_IGN);
  signal(SIGQUIT, fmQuit);
  signal(SIGHUP, forcedQuit);
  
  fm_white_pixel = exWhitePixel(display);
  fm_black_pixel = exBlackPixel(display);

  /* store the function defintions for use by the menu system */
  cnt = 0;
  MH_FUNCTIONS[cnt].func      = (int (*)()) 0;
  MH_FUNCTIONS[cnt].funcName  = (char *) 0; cnt++;
  MH_FUNCTIONS[cnt].func      = buildTree;
  MH_FUNCTIONS[cnt].funcName  = "build tree"; cnt++;
  MH_FUNCTIONS[cnt].func      = cleanupDirWindows;
  MH_FUNCTIONS[cnt].funcName  = "clean up"; cnt++;
  MH_FUNCTIONS[cnt].func      = cpFile;
  MH_FUNCTIONS[cnt].funcName  = "copy files"; cnt++;
  MH_FUNCTIONS[cnt].func      = delFile;
  MH_FUNCTIONS[cnt].funcName  = "delete files"; cnt++;
  MH_FUNCTIONS[cnt].func      = commit;
  MH_FUNCTIONS[cnt].funcName  = "commit"; cnt++;
  MH_FUNCTIONS[cnt].func      = editAttr; 
  MH_FUNCTIONS[cnt].funcName  = "edit attributes"; cnt++;
  MH_FUNCTIONS[cnt].func      = fmAbort;
  MH_FUNCTIONS[cnt].funcName  = "abort file manager"; cnt++;
  MH_FUNCTIONS[cnt].func      = fmQuit;
  MH_FUNCTIONS[cnt].funcName  = "leave file manager"; cnt++;
  MH_FUNCTIONS[cnt].func      = listDirAny;
  MH_FUNCTIONS[cnt].funcName  = "list any directory"; cnt++;
  MH_FUNCTIONS[cnt].func     = listDirFather;
  MH_FUNCTIONS[cnt].funcName = "list father directory"; cnt++;
  MH_FUNCTIONS[cnt].func     = listDirSel;
  MH_FUNCTIONS[cnt].funcName = "list selected directory"; cnt++;
  MH_FUNCTIONS[cnt].func     = mvFile;
  MH_FUNCTIONS[cnt].funcName = "move files"; cnt++;
  MH_FUNCTIONS[cnt].func     = remDirWnd;
  MH_FUNCTIONS[cnt].funcName = "remove window"; cnt++;
  MH_FUNCTIONS[cnt].func     = renFile;
  MH_FUNCTIONS[cnt].funcName = "rename files"; cnt++;
  MH_FUNCTIONS[cnt].func     = selAll;
  MH_FUNCTIONS[cnt].funcName = "select all"; cnt++;
  MH_FUNCTIONS[cnt].func     = sortByAccess;
  MH_FUNCTIONS[cnt].funcName = "order by access"; cnt++;
  MH_FUNCTIONS[cnt].func     = sortByMod;
  MH_FUNCTIONS[cnt].funcName = "order by mod"; cnt++;
  MH_FUNCTIONS[cnt].func     = sortByName;
  MH_FUNCTIONS[cnt].funcName = "order by name"; cnt++;
  MH_FUNCTIONS[cnt].func     = sortBySize;
  MH_FUNCTIONS[cnt].funcName = "order by size"; cnt++;
  MH_FUNCTIONS[cnt].func     = sortHorizontal;
  MH_FUNCTIONS[cnt].funcName = "horizontal"; cnt++;
  MH_FUNCTIONS[cnt].func     = sortVertical;
  MH_FUNCTIONS[cnt].funcName = "vertical"; cnt++;
  MH_FUNCTIONS[cnt].func     = undelFile;
  MH_FUNCTIONS[cnt].funcName = "undelete files"; cnt++;
  MH_FUNCTIONS[cnt].func     = unrenFile;
  MH_FUNCTIONS[cnt].funcName = "unrename files"; cnt++;
  MH_FUNCTIONS[cnt].func     = unselAll;
  MH_FUNCTIONS[cnt].funcName = "unselect all"; cnt++;
  MH_FUNCTIONS[cnt].func     = dupFile;
  MH_FUNCTIONS[cnt].funcName = "dupFile"; cnt++;
  MH_FUNCTIONS[cnt].func     = selByGrep;
  MH_FUNCTIONS[cnt].funcName = "selByGrep"; cnt++;
  MH_FUNCTIONS[cnt].func     = getFilePattern;
  MH_FUNCTIONS[cnt].funcName = "getFilePattern"; cnt++;
  MH_FUNCTIONS[cnt].func     = showFileAttributes;
  MH_FUNCTIONS[cnt].funcName = "showFileAttributes"; cnt++;
  MH_FUNCTIONS[cnt].func     = putCurrentNames;
  MH_FUNCTIONS[cnt].funcName = "putCurrentNames"; cnt++;
  MH_FUNCTIONS[cnt].func     = putAllNames;
  MH_FUNCTIONS[cnt].funcName = "putAllNames"; cnt++;
  MH_FUNCTIONS[cnt].func     = makeDir;
  MH_FUNCTIONS[cnt].funcName = "makeDir"; cnt++;
  MH_FUNCTIONS[cnt].func     = (int (*)()) 0;
  MH_FUNCTIONS[cnt].funcName = (char *) 0; cnt++;
  
  PanelCursor = XCreateFontCursor(display, XC_dot);
  TreeCursor  = XCreateFontCursor(display, XC_top_left_arrow);
  CopyCursor  = XCreateFontCursor(display, XC_question_arrow);

  depth = DefaultDepth(display, DefaultScreen(display)); 
  closeBoxPixmap = XCreatePixmapFromBitmapData(display, w, closeBox_bits,
      closeBox_width, closeBox_height, fm_black_pixel, fm_white_pixel, depth);

  iconPixmap = XCreatePixmapFromBitmapData(display, w, dirwIcon_bits,
      dirwIcon_width, dirwIcon_height, fm_black_pixel, fm_white_pixel, depth);

  if (makeFullName(getMenuDir(), FMMENUS, fullName, 256) < 0) {
    exError("aed", "cannot make full Name", NULL);
    return(-1);
  }
  
  if (restoreFromFile(parent, fullName, -1) == False) {
    exAlert(parent, 100, 100, "Cannot load menus !");
    return(-1);
  }

  fm_aptr = currentApplication();

  return(0);
}

/* initDir: ******************************************************************
 * create the first directory window and initialize the anchor of the list of 
 * displayed directory windows
 ****************************************************************************/

initDir(root, x, y, dir)
     Widget root;
     int x, y;
     char *dir;
{
  BufInfo *bufinfoptr();
  dirItem *items;
  int cnt;
  Arg args[10];
  GC gc;
  Display *d;
  dirWindow *wnd;
  XmFontList fl = NULL;
  
  d = XtDisplay(root);
  
  gc = XCreateGC(d, XDefaultRootWindow(d), 0, 0);
  XSetBackground(d, gc, exWhitePixel(d));
  XSetForeground(d, gc, exBlackPixel(d));
  XSetFunction(d, gc, GXcopy);
  XSetLineAttributes(d, gc, 1, LineSolid, CapButt, JoinMiter);

  Head = (dirWndHead *) Malloc(sizeof(dirWndHead));

  Head->fm_aptr = fm_aptr;
  Head->fm_gc = gc;
  Head->aed_form = exCreateAedForm(root, 100, 100, False);

  /* hide the top-level window */
  cnt = 0;
  XtSetArg(args[cnt], XmNx, 0); cnt++;
  XtSetArg(args[cnt], XmNy, 0); cnt++;
  XtSetArg(args[cnt], XmNwidth, 1); cnt++;
  XtSetArg(args[cnt], XmNheight, 1); cnt++;
  XtSetArg(args[cnt], XmNmappedWhenManaged, False); cnt++;
  XtSetValues(root, args, cnt);

  Head->root = root;
  Head->curx = x;
  Head->cury = y;
  
  createAttributeDisplay(Head);

  cnt = readDirectory(dir, &items);
  wnd = allocateDirWindow(dir, dir, items, cnt);

  /* try to get font from the created directory window */
  cnt = 0;
  XtSetArg(args[cnt], XmNtextFontList, &fl); cnt++;
  XtGetValues(wnd->fmWindow, args, cnt);
    
  if (! fl)
    Head->itemFont = exBaseFont(XtDisplay(Head->root));
  else
    Head->itemFont = getFont(fl);

  Head->mb_height = menuBarHeight(wnd->menubar);
  Head->sb_width = exFontHeight(Head->itemFont);
  Head->title_height = titleHeight(wnd->title);

  sortItemList(wnd);
  markVisibleItems(wnd, "*", False);
  displayDirWindow(wnd);

  Head->first = Head->last = wnd;
}

/* allocateDirWindow: *********************************************************
 * allocate a dirWindow structure and initialise all sub-structures
 *****************************************************************************/

dirWindow *allocateDirWindow(path, upath, list, icnt)
     char *path;	/* the pathname */
     char *upath;       /* the unix pathname */
     dirItem *list;	/* list of items */
     int icnt;		/* number of entries in directory  */
{
  dirItem *iptr;
  dirWindow *wnd;
  Arg args[20];
  int cnt;
  char pattern[200];

  static XtTranslations ptt = NULL, ptt1 = NULL;

  static char fm_translations[] = 
    " <Enter>     : updateIfNecessary() \n \
      <Configure> : fmRedraw()";

  static char work_translations[] = 
    " <Create>    : fmSetCursor()";

  static XtActionsRec fm_actions[] = {
    { "updateIfNecessary", updateIfNecessary},
    { "fmRedraw", fmRedraw},
    { "fmSetCursor", fmSetCursor}
  };

  if (! ptt) {
    XtAddActions(fm_actions, 3);
    ptt  = XtParseTranslationTable(fm_translations);
    ptt1 = XtParseTranslationTable(work_translations);
  }

  sprintf(pattern, "%s/*", upath);
  wnd = (dirWindow *) Malloc(sizeof(dirWindow));
  wnd->path = strCopy(path);
  wnd->u_path = strCopy(upath);	
  wnd->prev = (dirWindow *) NULL;
  wnd->next = (dirWindow *) NULL;
  wnd->itemList = list;
  wnd->tree = (treeWindow *) NULL;
  wnd->itemCnt = icnt;				
  wnd->w_lines = MIN(LINESPERWINDOW, ROUNDOFF(icnt, ITEMSPERLINE));
  wnd->w_cols = ITEMSPERLINE;
  wnd->flags = DIRW_VERT | DIRW_FIRST;
  wnd->sortFlag = ORDER_BYNAME;
  wnd->menubar = (exMenubar *) 0;
  wnd->workArea = (Widget) 0;
  wnd->filePattern = strCopy(pattern);

  
  /* top-level shell */
  cnt = 0;
  XtSetArg(args[cnt], XmNborderWidth, BORDERWIDTH); cnt++;
  XtSetArg(args[cnt], XmNx, Head->curx); cnt++;
  XtSetArg(args[cnt], XmNy, Head->cury); cnt++;
  XtSetArg(args[cnt], XmNkeyboardFocusPolicy, XmPOINTER); cnt++;

  wnd->fmTop = XtCreatePopupShell(VERSION, 
		     transientShellWidgetClass, Head->root, args, cnt);

  /* top-most form widget */
  cnt = 0;
  XtSetArg(args[cnt], XmNuserData, wnd); cnt++;
  wnd->fmWindow = 
    XtCreateManagedWidget("dir_box", xmFormWidgetClass, wnd->fmTop, args, cnt);

  XtOverrideTranslations(wnd->fmWindow, ptt);
  setReadTime(wnd);

  /* create close box */
  cnt = 0;
  XtSetArg(args[cnt], XmNrightAttachment, XmATTACH_FORM); cnt++;
  XtSetArg(args[cnt], XmNtopAttachment, XmATTACH_FORM); cnt++;
  XtSetArg(args[cnt], XmNlabelType, XmPIXMAP); cnt++;
  XtSetArg(args[cnt], XmNlabelPixmap, closeBoxPixmap); cnt++;
  XtSetArg(args[cnt], XmNmarginHeight, 0); cnt++;
  XtSetArg(args[cnt], XmNmarginWidth, 0); cnt++;

  wnd->closeBox = XtCreateManagedWidget("close_box", xmPushButtonWidgetClass,
				       wnd->fmWindow, args, cnt);
  XtAddCallback(wnd->closeBox, XmNactivateCallback, closeDirWindow, wnd);

  /*create title-bar */
  cnt = 0;
  XtSetArg(args[cnt], XmNrightAttachment, XmATTACH_WIDGET); cnt++;
  XtSetArg(args[cnt], XmNrightWidget, wnd->closeBox); cnt++;
  XtSetArg(args[cnt], XmNleftAttachment, XmATTACH_FORM); cnt++;
  XtSetArg(args[cnt], XmNtopAttachment, XmATTACH_FORM); cnt++;
  XtSetArg(args[cnt], XmNalignment, XmALIGNMENT_BEGINNING); cnt++;
  wnd->title = XtCreateManagedWidget("title", xmLabelWidgetClass,
				     wnd->fmWindow, args, cnt);

  /* make menubar and load menus */
  cnt = 0;
  XtSetArg(args[cnt], XmNrightAttachment, XmATTACH_FORM); cnt++;
  XtSetArg(args[cnt], XmNleftAttachment, XmATTACH_FORM); cnt++;
  XtSetArg(args[cnt], XmNtopAttachment, XmATTACH_WIDGET); cnt++;
  XtSetArg(args[cnt], XmNtopWidget, wnd->title); cnt++;
    
  wnd->menubar = 
    exCreateMenuBar(wnd->fmWindow, args, cnt, NULL);
  loadMenus(wnd->menubar, wnd->fmWindow, Head->fm_aptr, (MenuDescr *)NULL, 
	    (int *)NULL);
  
  /* make work area */
  cnt = 0;
  XtSetArg(args[cnt], XmNtopAttachment, XmATTACH_WIDGET); cnt++;
  XtSetArg(args[cnt], XmNtopWidget, wnd->menubar->menubar); cnt++;
  XtSetArg(args[cnt], XmNleftAttachment, XmATTACH_FORM); cnt++;
  XtSetArg(args[cnt], XmNrightAttachment, XmATTACH_FORM); cnt++;
  XtSetArg(args[cnt], XmNbottomAttachment, XmATTACH_FORM); cnt++;
  XtSetArg(args[cnt], XmNmarginHeight, 0); cnt++;
  XtSetArg(args[cnt], XmNmarginWidth, 0); cnt++;
  XtSetArg(args[cnt], XmNuserData, wnd); cnt++;
  
  wnd->workArea = XtCreateManagedWidget("work", xmFormWidgetClass,
					wnd->fmWindow, args, cnt);
  
  XtOverrideTranslations(wnd->workArea, ptt1);

  for (iptr = list; iptr; iptr = iptr->next)
    iptr->dirWnd = wnd;

  Head->cury += 100;

  return((dirWindow *) wnd);
}

/* removeDirWindow: ***********************************************************
 * remove the directory-window from the screen 
 ****************************************************************************/

removeDirWindow(dirw)
     dirWindow *dirw;
{
  /* dequeue dirWindow */
  if (dirw->prev == (dirWindow *)NULL && dirw->next == (dirWindow *)NULL) {
    exAlert(Head->root, 100, 100, "Cannot delete the last directory window");
    return;
  }

  if (dirw->prev == (dirWindow *) NULL) { 	/* first in chain */
    Head->first = dirw->next;
    dirw->next->prev = (dirWindow *) NULL;
  } else if (dirw->next == (dirWindow *) NULL) { /* last in chain */
    dirw->prev->next = (dirWindow *) NULL;
    Head->last = dirw->prev;
  } else {					 /* in between */
    dirw->prev->next = dirw->next;
    dirw->next->prev = dirw->prev;
  }

  deleteTreeWindow(dirw);
  freeDirWnd(dirw);
}

/* makeItems: ***************************************************************
 * generate the menu-panel from the list of items 
 ****************************************************************************/

makeItems(wnd)
     dirWindow *wnd;
{
  MHmenu *mptr;
  int cnt;
  dirItem *dptr;
  register MHitem *iptr;
  char s[10], name[MAXITEMLEN + 1];
  ApplicationNode *aptr = currentApplication();
  
  setWorkApplication();
  
  if ((mptr = createMenu()) == (MHmenu *) 0) {	   	/* make the menu */
    setError(ERR_CREATE);
    setCurrentApplication(aptr);
    return(-1);
  }

  insertMenu(mptr);
  initMenu(mptr, MDS_L_POPUP, wnd->path);
  setSelCnt(mptr, 0);
  setSelType(mptr, MDS_CNT_GE);
  mptr->m_hlp->hlpmsg = strCopy("Please select files.");
  mptr->m_key = EOS;
  mptr->maxItem = LINESPERWINDOW;
  
  if (wnd->flags & DIRW_VERT || wnd->w_cols == 1) { 	/* vertically */
    for (cnt = 0, dptr = wnd->itemList; dptr; dptr = dptr->next) {
      if (! (dptr->flags & ITEM_VISIBLE)) {
	dptr->slot = -1;
	continue;
      }
      
      sprintf(name, "      %-.ITEMLEN", dptr->D_name);
      if (strlen(name) < MAXITEMLEN) 
	strncat(name, "               ", MAXITEMLEN - strlen(name));

      iptr = newItem(mptr, name, EOS);			/* create item */
      sprintf(s, "%d", cnt);
      setRetPattern(iptr, s);
      dptr->slot = cnt;
      cnt++;
    }
  } else {						/* horizontally */ 
    dirItem **itemTable;
    int indx, i, j, not_filled, itemsPerCol, lines;

    /* count visible items */
    for (cnt = 0, dptr = wnd->itemList; dptr; dptr = dptr->next)
      if (dptr->flags & ITEM_VISIBLE)
	cnt++;

    if (cnt <= 0) {					/* no items visible */
      setCurrentApplication(aptr);
      return(-1);
    }

    itemsPerCol = ROUNDOFF(cnt, wnd->w_cols);
    not_filled = (wnd->w_cols - (cnt % wnd->w_cols)) % wnd->w_cols;
    itemTable = (dirItem **) calloc(cnt, sizeof(dirItem *));
    for (dptr = wnd->itemList;				/* 1st visible item */
	 ! (dptr->flags & ITEM_VISIBLE); dptr = dptr->next);
    lines = 0;

    for (i = 0; i < itemsPerCol && dptr; i++) {
      for (j = 0; j < wnd->w_cols; j++) {
	indx = j * itemsPerCol + i;
	if (indx < cnt) {
	  itemTable[indx]  = dptr; 
	  for (dptr = dptr->next; 			/* next visible item */
	       dptr && !(dptr->flags & ITEM_VISIBLE); dptr = dptr->next);
	  if (lines > itemsPerCol - not_filled)     	/* not fill last row */
	    if (j >= wnd->w_cols - 1)
	      break;
	}
      }
      lines++;						/* next item line */
    }

    for (i = 0; i < cnt; i++)				/* create items */
      if (itemTable[i]) {
	itemTable[i]->slot = i;

	sprintf(name, "      %-.ITEMLEN", itemTable[i]->D_name);
	if (strlen(name) < MAXITEMLEN) 
	  strncat(name, "               ", MAXITEMLEN - strlen(name));

	iptr = newItem(mptr, name, EOS);
	sprintf(s, "%d", i);
	setRetPattern(iptr, s);
      }
    free(itemTable);
  }

  wnd->m = exMakePanel((ApplicationNode *)NULL, wnd->workArea, mptr, 
		       wnd->w_cols, -1, True);

  setCurrentApplication(aptr);

  return(0);
}

/* allocateDirItem: ***********************************************************
 * allocate a item-structure and initialise some sub-compoents
 *****************************************************************************/

dirItem *allocateDirItem()
{
  register dirItem *iptr;

  iptr = (dirItem *) Malloc(sizeof(dirItem));
  iptr->next = (dirItem *) NULL;
  iptr->a_name = (char *) NULL;
  iptr->D_name = (char *) NULL;
  iptr->sbp    = (struct stat *) NULL;
  iptr->flags = 0;
  iptr->dirWnd = (dirWindow *) NULL;

  return((dirItem *) iptr);
}

/* freeItems: *****************************************************************
 * free the space occupied by the specified items
 *****************************************************************************/

freeItems(item_list)
     dirItem *item_list;
{
  register dirItem *iptr, *optr;
  
  for (iptr = item_list; iptr; ) {
    free(iptr->sbp);
    free(iptr->D_name);
    if (iptr->a_name)
      free(iptr->a_name);
    optr = iptr;
    iptr = iptr->next;
    free(optr);
  }
} 

/* freeDirWnd: ****************************************************************
 * free the dirWindow data-structure 
 *****************************************************************************/

freeDirWnd(dirw)
     dirWindow *dirw;
{
  freeItems(dirw->itemList);

  free(dirw->filePattern);
  free(dirw->path);
  free(dirw->u_path);

  XtDestroyWidget(dirw->fmTop);

  free(dirw);
}

/* makeDirNodeStruct: *********************************************************
 * allocate the structure that contains the information pertaining to the node 
 * of a directory tree
 *****************************************************************************/

dirNode *makeDirNodeStruct(tw, col, row, path)
     treeWindow *tw;
     int col, row;
     char *path;
{
  dirNode *b;

  b = (dirNode *) Malloc(sizeof(dirNode));

  b->x = (col - 1) * (DIRBUTTONWIDTH(tw->font) + DIRBUTTONHGAP);
  b->y = (row - 1) * (DIRBUTTONHEIGHT(tw->font) + DIRBUTTONVGAP);
  b->name = strCopy(path);

  b->next = tw->nodes;				/* put node in list */
  tw->nodes = b;
  
  return(b);
}

/* defineIcon: ***************************************************************
 * create an icon window for the given directory window 
 ****************************************************************************/

void defineIcon(wnd)
     dirWindow *wnd;
{
  Arg args[10];
  int cnt;
  Position x, y;

  /* create icon shell */
  cnt = 0;
  XtSetArg(args[cnt], XmNx, &x); cnt++;
  XtSetArg(args[cnt], XmNy, &y); cnt++;
  XtGetValues(wnd->fmTop, args, cnt);

  cnt = 0;
  XtSetArg(args[cnt], XmNx, x); cnt++;
  XtSetArg(args[cnt], XmNy, y); cnt++;
  XtSetArg(args[cnt], XmNkeyboardFocusPolicy, XmPOINTER); cnt++;

  wnd->dirwIcon.iconShell =  
    XtCreatePopupShell("icon shell", overrideShellWidgetClass, 
		       				Head->root, args, cnt);
  /* create icon window */
  cnt = 0;
  XtSetArg(args[cnt], XmNwidth, dirwIcon_width); cnt++;
  XtSetArg(args[cnt], XmNheight, 
	   dirwIcon_height + closeBox_height + 2 + 2 * 2); cnt++;
  XtSetArg(args[cnt], XmNkeyboardFocusPolicy, XmPOINTER); cnt++;

  wnd->dirwIcon.icon = 
    XtCreateManagedWidget("icon", 
		    xmFormWidgetClass, wnd->dirwIcon.iconShell, args, cnt);

  /* create icon close box */
  cnt = 0;
  XtSetArg(args[cnt], XmNrightAttachment, XmATTACH_FORM); cnt++;
  XtSetArg(args[cnt], XmNtopAttachment, XmATTACH_FORM); cnt++;
  XtSetArg(args[cnt], XmNlabelType, XmPIXMAP); cnt++;
  XtSetArg(args[cnt], XmNlabelPixmap, closeBoxPixmap); cnt++;
  XtSetArg(args[cnt], XmNborderWidth, 2); cnt++;
  XtSetArg(args[cnt], XmNmarginHeight, 0); cnt++;
  XtSetArg(args[cnt], XmNmarginWidth, 0); cnt++;

  wnd->dirwIcon.iconCloseBox = 
    XtCreateManagedWidget("close_box", xmPushButtonWidgetClass,
				      wnd->dirwIcon.icon, args, cnt);
  XtAddCallback(wnd->dirwIcon.iconCloseBox, 
		XmNactivateCallback, openDirWindow, wnd); 

  /*create title-bar */
  cnt = 0;
  XtSetArg(args[cnt], XmNrightAttachment, XmATTACH_WIDGET); cnt++;
  XtSetArg(args[cnt], XmNrightWidget, wnd->dirwIcon.iconCloseBox); cnt++;
  XtSetArg(args[cnt], XmNleftAttachment, XmATTACH_FORM); cnt++;
  XtSetArg(args[cnt], XmNtopAttachment, XmATTACH_FORM); cnt++;
  XtSetArg(args[cnt], XmNalignment, XmALIGNMENT_END); cnt++;
  XtSetArg(args[cnt], XmNborderWidth, 1); cnt++;
  wnd->dirwIcon.iconTitle = XtCreateManagedWidget("title", 
		 xmLabelWidgetClass, wnd->dirwIcon.icon, args, cnt);
  setTitle(wnd->dirwIcon.iconTitle, wnd->path);

  /* create icon pixmap area */
  cnt = 0;
  XtSetArg(args[cnt], XmNrightAttachment, XmATTACH_FORM); cnt++;
  XtSetArg(args[cnt], XmNleftAttachment, XmATTACH_FORM); cnt++;
  XtSetArg(args[cnt], XmNtopAttachment, XmATTACH_WIDGET); cnt++;
  XtSetArg(args[cnt], XmNtopWidget, wnd->dirwIcon.iconCloseBox); cnt++;
  XtSetArg(args[cnt], XmNbottomAttachment, XmATTACH_FORM); cnt++;
  XtSetArg(args[cnt], XmNwidth, dirwIcon_width); cnt++; 
  XtSetArg(args[cnt], XmNheight, dirwIcon_height); cnt++; 
  wnd->dirwIcon.iconWindow = XtCreateManagedWidget("area", 
		 xmDrawingAreaWidgetClass, wnd->dirwIcon.icon, args, cnt);
  XtAddCallback(wnd->dirwIcon.iconWindow, XmNexposeCallback, redrawIcon, wnd);
}

/* forcedQuit: ***************************************************************
 * this function is called upon receipt of a SIGHUP signal from the eshell in
 * order to terminate; 
 *****************************************************************************/

static forcedQuit()
{
  register dirWindow *w;

  for (w = Head->first; w; w = w->next) 
    commitChanges(w, True, False);
    
  quitFm((dirWindow *)NULL, FMABORT);
}

/* updateIfNecessary: ********************************************************
 * 
 *****************************************************************************/

static void updateIfNecessary(w, event, params, num_params)
     Widget w;
     XLeaveWindowEvent *event;
     String *params;
     Cardinal *num_params;
{ 
  dirWindow *wnd;
  Arg args[1];
  int cnt;
  
  cnt = 0;
  XtSetArg(args[cnt], XmNuserData, &wnd); cnt++;
  XtGetValues(w, args, cnt);

  if (! wnd)
    return;

  if (needUpdate(wnd))
    updateDirWindow(wnd);
}

/* createAttributeDisplay: ****************************************************
 * create the editor to display all attributes of selected files
 ****************************************************************************/

static void createAttributeDisplay(head)
  dirWndHead *head;
{
  int cnt;
  EditorDescr descr;
  Arg args[10];
  BufInfo *bufinfoptr();
  void hideAttrDisplay();

  descr.inputFnct = (int (*)()) 0;
  descr.file = (char *) 0;
  descr.name = "file attributes";
  descr.buffer = (char **) 0;
  descr.bufferLines = 200;
  descr.lines = 24;
  descr.cols = 80;
  descr.flags = ED_MSGWINDOW | ED_SCROLLBAR | ED_HELP | ED_AUTOKILL;

  /* attribute display top-level shell */
  cnt = 0;
  XtSetArg(args[cnt], XmNx, 100); cnt++;
  XtSetArg(args[cnt], XmNy, 100); cnt++;
  XtSetArg(args[cnt], XmNkeyboardFocusPolicy, XmPOINTER); cnt++;

  head->attrShell = XtCreatePopupShell("attribute_display", 
			 transientShellWidgetClass, head->root, args, cnt);

  /* attribute display main window */
  cnt = 0;
  XtSetArg(args[cnt], XmNborderWidth, 2); cnt++;
  head->attrWindow = XtCreateManagedWidget("attribute", 
			      xmFormWidgetClass, head->attrShell, args, cnt);

  /* create close box */
  cnt = 0;
  XtSetArg(args[cnt], XmNrightAttachment, XmATTACH_FORM); cnt++;
  XtSetArg(args[cnt], XmNtopAttachment, XmATTACH_FORM); cnt++;
  XtSetArg(args[cnt], XmNlabelType, XmPIXMAP); cnt++;
  XtSetArg(args[cnt], XmNlabelPixmap, closeBoxPixmap); cnt++;
  XtSetArg(args[cnt], XmNmarginHeight, 0); cnt++;
  XtSetArg(args[cnt], XmNmarginWidth, 0); cnt++;

  head->attrCloseBox = XtCreateManagedWidget("close_box", 
		 xmPushButtonWidgetClass, head->attrWindow, args, cnt);
  XtAddCallback(head->attrCloseBox, XmNactivateCallback, hideAttrDisplay, 
		head);

  /* attribute display title */
  cnt = 0;
  XtSetArg(args[cnt], XmNtopAttachment, XmATTACH_FORM); cnt++;
  XtSetArg(args[cnt], XmNleftAttachment, XmATTACH_FORM); cnt++;
  XtSetArg(args[cnt], XmNrightAttachment, XmATTACH_WIDGET); cnt++;
  XtSetArg(args[cnt], XmNrightWidget, head->attrCloseBox); cnt++;
  XtSetArg(args[cnt], XmNlabelString, 
	   XmStringCreate(descr.name, XmSTRING_DEFAULT_CHARSET)); cnt++; 

  head->attrTitle = XtCreateManagedWidget("title", 
		       xmLabelWidgetClass, head->attrWindow, args, cnt);

  /* attr. editor msg area */
  cnt = 0;
  XtSetArg(args[cnt], XmNbottomAttachment, XmATTACH_FORM); cnt++;
  XtSetArg(args[cnt], XmNleftAttachment, XmATTACH_FORM); cnt++;
  XtSetArg(args[cnt], XmNrightAttachment, XmATTACH_FORM); cnt++;
  
  descr.msg = XtCreateManagedWidget("msg", xmDrawingAreaWidgetClass,
			  head->attrWindow, args, cnt);
  
  /* attr. editor working area */
  cnt = 0;
  XtSetArg(args[cnt], XmNtopAttachment, XmATTACH_WIDGET); cnt++;
  XtSetArg(args[cnt], XmNtopWidget, head->attrTitle); cnt++;
  XtSetArg(args[cnt], XmNleftAttachment, XmATTACH_FORM); cnt++;
  XtSetArg(args[cnt], XmNrightAttachment, XmATTACH_FORM); cnt++;
  XtSetArg(args[cnt], XmNbottomAttachment, XmATTACH_WIDGET); cnt++;
  XtSetArg(args[cnt], XmNbottomWidget, descr.msg); cnt++;

  head->attrEditor = makeEditor(head->attrWindow, "attr", &descr, args, cnt);
  initCmdTables(head->attrEditor, CmdTable, EscCmds, ctrlXCmds, fmCmds);

  /* set height of message area */
  cnt = 0;
  XtSetArg(args[cnt], XmNheight, 
	  3 * exFontHeight((bufinfoptr(head->attrEditor))->font) / 2); cnt++;
  XtSetValues(descr.msg, args, cnt);
}

/* setDirBoxSizeHints: *******************************************************
 * calculate and set the resize hints for the directory window;
 * the calculations assume that all entries have the same font and therefore
 * have identical widths and heights.
 *****************************************************************************/

setDirBoxSizeHints(wnd)
     dirWindow *wnd;
{
  XSizeHints size_hints;
  int i, cnt, item_gap;
  int sb_width;
  Dimension width = 0, height = 0;
  Arg args[2];

  if (wnd->m->flags & SCROLLABLE)
    sb_width = Head->sb_width;
  else 
    sb_width = 0;

  item_gap = exTextWidth(wnd->m->itemFont, "   ");	/* see exPanelLayout */

  size_hints.flags = PMinSize | PResizeInc | PMaxSize;
  
  if (wnd->m->itemCnt < 1) { 			/* no items yet */
    size_hints.min_height = Head->mb_height + Head->title_height	
				         + exFontHeight(Head->itemFont);
    size_hints.min_width  = mbWidth(wnd->menubar);

    size_hints.max_height = size_hints.min_height;
    size_hints.max_width  = size_hints.min_width;

    size_hints.width_inc  = 0;
    size_hints.height_inc = 0;
  } else {
    size_hints.min_width = sb_width + wnd->m->entries[0]->width;
    
    size_hints.min_height = Head->mb_height + Head->title_height +
      wnd->m->entries[0]->ypos + wnd->m->entries[0]->height;
    
    i = ROUNDOFF(wnd->m->itemCnt, wnd->m->cols);
    
    size_hints.max_width  = MAX(mbWidth(wnd->menubar),
				sb_width + wnd->m->entries[0]->xpos + 
	wnd->m->cols * wnd->m->entries[0]->width + wnd->m->cols * item_gap);
    
    if (i < 1) 					/* at most one row of items */
      size_hints.max_height = size_hints.min_height;
    else 
      size_hints.max_height = Head->mb_height + Head->title_height +
					wnd->m->entries[0]->ypos +
	  				i * wnd->m->entries[0]->height;
    
    size_hints.width_inc  = wnd->m->entries[0]->width + item_gap;
    size_hints.height_inc = wnd->m->entries[0]->height;
  }

  XSetNormalHints(XtDisplay(wnd->fmTop), XtWindow(wnd->fmTop), &size_hints);

  /* resize if window should shrink because of fewer items */

  cnt = 0;
  XtSetArg(args[cnt], XmNwidth, &width); cnt++;
  XtSetArg(args[cnt], XmNheight, &height); cnt++;
  XtGetValues(wnd->fmTop, args, cnt);		/* get actual size */

  if (height > size_hints.max_height || width > size_hints.max_width)
    XtResizeWidget(wnd->fmTop, 
		   MIN(width, size_hints.max_width),
		   MIN(height, size_hints.max_height), 
		   BORDERWIDTH);
}
