/****************************************************************************
 * contains command tables for the history tracers editor
 ****************************************************************************/

static editCmd histEditCmdTable [] = { 	/* define standard command set */
  {ctrl('a'), hBeglin},     		/* beginning of line */
  {ctrl('b'), hLeft},          		/* cursor left  */
  {ctrl('d'), hDelnxt},	                /* delete next character */
  {ctrl('e'), hEndlin},     		/* end of line */
  {ctrl('f'), hRight},          	/* cursor right */
  {ctrl('j'), hNewline},                /* insert carriage return */
  {ctrl('k'), hKrest},			/* delete to end of line */
  {ctrl('l'), hNewscr},               	/* redraw screen */
  {ctrl('m'), hNewline},                /* insert carriage return */
  {ctrl('n'), hDwncol},     		/* down in column */
  {ctrl('o'), hOpenln},	                /* open line above */
  {ctrl('p'), hUpcol},     		/* up in column */
  {ctrl('r'), hBwsearch},	        /* search backward */
  {ctrl('s'), hFsearch},		/* search forward */
  {ctrl('v'), hDwnpage},       		/* forward page */
  {ctrl('w'), hCut},			/* cut region into kill buffer */
  {ctrl('y'), hYank},       		/* yank killbuffer */
  {127,     hDelprev},            	/* delete previous char */
  {'\n',    hNewline},                  /* insert carriage return */
  {'\r',    hNewline},                  /* insert carriage return */
  {EOS,     (int (*)()) 0},		/* end of list */
};		

static editCmd histCtrlXCmds [] = {	
  {ctrl('c'), hEditExit},               /* leave editor */
  {ctrl('d'), dot},			/* set dot to current position */
  {ctrl('f'), hFindFile},		/* read in alternate file */
  {ctrl('i'), hInsertFile},		/* insert named file into buffer */
  {ctrl('k'), clearBuffer},		/* kill the current buffer */
  {ctrl('m'), exMark},			/* set mark to current position */
  {ctrl('s'), hSaveFile},		/* save buffer to current file */
  {ctrl('w'), hWriteFile},		/* write buffer to named file */
  {ctrl('x'), exchangeDotMark},		/* exchange dot and mark */
  {ctrl('l'), execLastCom},             /* execute last command from history */
  {ctrl('n'), execNextCom},             /* execute next command relative to
					   history pointer */
  {ctrl('p'), execPrevCom},             /* execute command above 
					   history pointer */
  {ctrl('t'), execThisCom},             /* execute command in which cursor 
                                           is positioned */
  {ctrl('h'), setHistPtr},              /* set internal history pointer */
  {ctrl('j'), joinWithNextCom},         /* join current command with next 
					   command */
  {'c', hEditExit},			/* leave editor */
  {'k', hKillBuffer},			/* clear the current buffer */
  {'o', hOpenlnd},	        	/* open line below */
  {'t', hLineToTop},			/* current line to top of screen */
  {'?', help},				/* display help information */
  {EOS, (int (*)()) 0},
};

static editCmd histEditEscCmds [] = {
  {'i', insertCom},	                /* insert new command */
  {'<', hTopBuffer},			/* goto top of file */
  {'>', hBotBuffer},			/* goto bottom of file */
  {' ', hGoLine},			/* goto specified line */
  {'=', memory},			/* display statistics */
  {'b', hWordl},			/* move to left word */
  {'d', hFdelword},			/* delete next word */
  {'f', hWordr},			/* move to right word */
  {'h', hTopscr},			/* cursor to top of display */
  {'l', hBotscr},			/* cursor to last line of display */
  {'m', hMidscr},			/* cursor near middle of display */
  {'n', hUpScroll},			/* scroll up */
  {'p', hDwnScroll},			/* scroll down */
  {'r', hBbsearch},			/* continue search backword */
  {'s', hFfsearch},			/* continue search forward */
  {'v', hUppage},			/* display previous page */
  {'w', hFdelword},			/* delete next word */
  {'?', whatLine},			/* display current line number */
  {127, hBdelword},			/* delete previous word */
  {ctrl('w'), hFdelword},		/* delete next word */
  {ctrl('x'), resetRegion},		/* display region not inverted */
  {EOS, (int (*)()) 0},
};

static cmdDescr htCmds[] = {
  {hRight, "n cursor right"},
  {hLeft, "n cursor left"},
  {hDwncol, "n down in column"},
  {hUpcol, "n up in column"},
  {hBeglin, "  beginning of line"},
  {hEndlin, "  end of line"},
  {hDwnpage, "n page back"},
  {hDelnxt, "n delete next character"},
  {hDelprev, "n delete previous char"},
  {hKrest, "  delete to end of line"},
  {hOpenln, "  open line above"},
  {hNewscr, "  redraw screen"},
  {hFsearch, "  search forward"},
  {hBwsearch,  "  search backward"},
  {hFindFile, "  edit another file"},
  {hSaveFile, "  save the current buffer"},
  {hWriteFile, "  write the current buffer to a file"},
  {hInsertFile, "  insert a file"},
  {execPrevCom, "  Provide the previous command relative to the history cursor."},
  {execNextCom, "  Provide the next command relative to the history cursor."},
  {execLastCom, "  Provide the last command in the history record."}, 
  {execThisCom, "  Provide the current command, i.e. the command to which the history cursor points."},
  {setHistPtr, "  Set the internal history cursor to the command where the text cursor is located."}, 
  {joinWithNextCom, "  join current command with next command"},
  {insertCom, "  insert new command"},
  {hEditExit, "  leave editor"},
  {hKillBuffer, "  abort the currrent edit buffer"},
  {hLineToTop, "  this line to top"},
  {hTopBuffer, "  first line in Buffer"},
  {hBotBuffer, "  last line in Buffer"},
  {hGoLine, "  goto line"},
  {hWordr, "n word forward"},
  {hWordl, "n word backward"},
  {hUpScroll, "n scroll up"},
  {hDwnScroll, "n scroll down"},
  {hUppage, "n forward page"},
  {hBotscr, "  last line of display"},
  {hMidscr, "  center of display"},
  {hTopscr, "  first line of display"},
  {hFdelword, "n delete next word"},
  {hFfsearch, "  continue search forward"},
  {hBbsearch, "  continue search backward"},
  {whatLine, "  show current line number "},
  {hBdelword, "n delete previous word"},
};

XtActionsRec dialogCmds[] = {
  {"hCursorPos",    (XtActionProc) hCursorPos},
  {"hCursorSelect", (XtActionProc) hCursorSelect},
  {"hCursorCopy",   (XtActionProc) hCursorCopy},
  {"hCursorCut",    (XtActionProc) hCursorCut},
  {"hCursorPaste",  (XtActionProc) hCursorPaste},
  {"hCursorDeselect",  (XtActionProc) hCursorDeselect},
};

char dialogCmdTable[] = 
  "Shift<Btn1Down>:  hCursorCopy()\n \
   <Btn1Down>:       hCursorPos()\n \
   Shift<Btn2Down>:  hCursorCut()\n \
   <Btn2Down>:       hCursorSelect()\n \
   Shift<Btn3Down>:  hCursorPaste()\n \
   <Btn3Down>     :  hCursorDeselect()";
;








