/************************************************************************
 *                                                                      *
 *    Copyright (c) 1991 Wilhelm Valder, Regine Freitag, Wolfgang Dzida *
 *                        All rights reserved                           *
 *                                                                      *
 *                                                                      *
 *    Permission to copy this program or any part of this program is    *
 *    granted in case of personal, non-commercial use only. Any use     *
 *    for profit or other commercial gain without written permissions   *
 *    of the authors is prohibited.                                     *
 *                                                                      *
 *    You may freely distribute this program to friends, colleagues,    *
 *    etc.  As a courtesy we would appreciate it if you left our names  *
 *    on the code as the authors.                                       *
 *    We also, of course, disavow any liabilities                       *
 *    for any bizarre things you may choose to do with the system.      *
 *                                                                      *
 *    We would appreciate though your letting us know about any         *
 *    extensions, modifications, project suggestions, etc.              *
 *    Please mail all bugs, comments, complaints and suggestions to:    *
 *                                                                      *
 *                                                                      *
 *    GMD (German National Research Center for Computer Science)        *
 *    Institut fuer Systemtechnik (F2G2)                                *
 *    P.O. Box 1240                                                     *
 *    Schloss Birlinghoven                                              *
 *    D-5205 St. Augustin 1, West Germany                               *
 *                                                                      *
 *    email:    dzida@kmx.gmd.dbp.de                                    *
 *              freitag@gmdzi.uunet                                     *
 *                                                                      *
 ************************************************************************/
#include <stdio.h>
#include <exBasics.h>
#include "exErrors.h"

int _mh_error;		/* global variable that holds the error-number */

#define _mh_max_error  (sizeof(_mh_errors) / sizeof(char *))

static char *_mh_errors[] = {
  "no error",
  "Internal error: illegal error number encountered",
  "Internal maximum exceeded",
  "Internal minimum exceeded",
  "Cannot close file",
  "Cannot open file with given access rights",
  "Cannot read (no read permission ?)",
  "Cannot write (no write permission ?)",
  "Illegal value encountered",
  "Illegal type encountered",
  "Illegal size encountered",
  "Null pointer encountered",
  "Function aborted",
  "Cannot allocate storage",
  "Cannot create",
  "Does not exist",
  "Does already exist",
  "Initialisation failed",
  "Specified access not allowed",
  (char *) 0
   };

/* uim_error: ****************************************************************
 * report an error as specified in the parameter error; the global variable 
 * _mh_error is set to indicate the error and the appropriate error message 
 * is printed; ERROR is returned to indicate that an error has occurred.
 ****************************************************************************/

uim_error(errno)
     int errno;
{
  char str[100];
  setError(errno);	/* set the global error variable */

  if (_mh_error < 0 || _mh_error > _mh_max_error)
    sprintf(str, "%s: %s\n", "reportError", _mh_errors[ERR_ERROR]);
  else 
    sprintf(str, "%s: %s\n", "mh error", _mh_errors[_mh_error]);
  
  fprintf(stderr, str);
  return(ERROR);
}

/* setError: ******************************************************************
 * the function setError sets the global variable _mh_error to the value
 * specified; it can then be used to report an error message via reportError()
 *****************************************************************************/

setError(err)
int err;
{
  if (err < 0 || err > _mh_max_error)
    _mh_error = ERR_ERROR;
  else
    _mh_error = err;
  
  return(_mh_error);
}


