/************************************************************************
 *                                                                      *
 *    Copyright (c) 1991 Wilhelm Valder, Regine Freitag, Wolfgang Dzida *
 *                        All rights reserved                           *
 *                                                                      *
 *                                                                      *
 *    Permission to copy this program or any part of this program is    *
 *    granted in case of personal, non-commercial use only. Any use     *
 *    for profit or other commercial gain without written permissions   *
 *    of the authors is prohibited.                                     *
 *                                                                      *
 *    You may freely distribute this program to friends, colleagues,    *
 *    etc.  As a courtesy we would appreciate it if you left our names  *
 *    on the code as the authors.                                       *
 *    We also, of course, disavow any liabilities                       *
 *    for any bizarre things you may choose to do with the system.      *
 *                                                                      *
 *    We would appreciate though your letting us know about any         *
 *    extensions, modifications, project suggestions, etc.              *
 *    Please mail all bugs, comments, complaints and suggestions to:    *
 *                                                                      *
 *                                                                      *
 *    GMD (German National Research Center for Computer Science)        *
 *    Institut fuer Systemtechnik (F2G2)                                *
 *    P.O. Box 1240                                                     *
 *    Schloss Birlinghoven                                              *
 *    D-5205 St. Augustin 1, West Germany                               *
 *                                                                      *
 *    email:    dzida@kmx.gmd.dbp.de                                    *
 *              freitag@gmdzi.uunet                                     *
 *                                                                      *
 ************************************************************************/
#include <exBasics.h>
#include <X11/Intrinsic.h>

#include "exEdit.h"

static char SccsID[] = "@(#)edtext.c	1.2\t11/10/92";            /* sccs keyword */

static char DELIMS[] = {' ', '\n', '\r', '\t', ';', ')', '(', '{', '}', 
			  ',', '<', '>', '!', '#', ':', '?', EOS};

static char *blankline = "";		/* blank lines point here */

void ETclear();
void ETunchanged();
BufInfo *bufinfoptr();
void autodeleteLines();
void ETdellines();			/* delete a number of lines */

Boolean ETappend();

/* growline: *****************************************************************
 * allocate or reallocate memory for line of text
 *****************************************************************************/

static Boolean growline(bf, line, len)
     BufInfo *bf;			/* pointer to buffer structure */
     int line;				/* line number */
     int len;				/* required lenght (will round up) */
{
  register Buffer *b = bf->bufPtr;

  len = (( len + ALLOCUNIT) / ALLOCUNIT) * ALLOCUNIT;
  if (b->text[line] == NULL || b->text[line] == blankline) {
    if ((b->text[line] = Malloc(len)) != NULL)	b->text[line][0] = '\0';
  }
  else
    b->text[line] = (char *) realloc(b->text[line], (unsigned)len);
  if(b->text[line] == NULL) {
    b->text[line] = blankline; 	/* must point to something! */
    return(False);
  }
  return(True);
}

/* freeline: *****************************************************************
 * free memory for line if it points to allocated memory
 ****************************************************************************/

static void freeline(b, line)
     register Buffer *b;		/* pointer to buffer structure */
     int line;				/* line number */
{
  if (b->text[line] != NULL && b->text[line] != blankline) {
    free(b->text[line]);
    b->text[line] = NULL;
  }
}

/* store: ********************************************************************
 * store line of text into buffer memory
 *****************************************************************************/

static Boolean store(b, line, text)
     register Buffer *b; 		/* pointer to buffer structure */
     int line; 				/* line number */
     char *text; 			/* text */
{
  b->changed = True;
  if (text[0] == '\0')
    b->text[line] = blankline;
  else if ((b->text[line] = Malloc(strlen(text) + 1)) == NULL) {
    b->text[line] = blankline;
    return(False);
  } else
    strcpy(b->text[line], text);
  return(True);
}

/* ETbegin*********************************************************************
 * initialize text buffer
 *****************************************************************************/

void ETbegin(editor, descr)
     EditorDescr *descr;
     Editor editor;
{
  register BufInfo *bf = bufinfoptr(editor);
  Buffer *b;
  register i;

  b = (Buffer *) Malloc(sizeof(Buffer));

  bf->bufPtr = b;
  b->numlines = 0;

  if (descr->flags & ED_EXTERNBUFFER) {	/* buffer provided by application */
    b->text = descr->buffer;
    for (i = 0; i < descr->bufferLines && b->text[i]; i++)
      b->numlines++;
  } else				/* generate buffer */
    b->text = (char **) Malloc(descr->bufferLines * sizeof(char *));

  b->bufferLines = descr->bufferLines;	/* size of text buffer */

  if (b->numlines == 0)
    (void)ETappend(bf, "");
  PSsetcur(bf, 0, 0);			/* home cursor */
  csrChrForget(bf);
  ETunchanged(bf);

  exEditSetScrollbar(bf);
}

/* ETclear: ******************************************************************
 * clear out buffer completely
 *****************************************************************************/

void ETclear(bf)
     BufInfo *bf;
{
  register i;
  register Buffer *b = bf->bufPtr;

  for (i = 0; i < b->numlines; i++)
    freeline(b, i);
  b->numlines = 0;
  b->changed = False;
}

/****************************************************************************
 * ETunchanged - turn off buffer's "changed" flag
 ***************************************************************************/

void ETunchanged(bf)
     BufInfo *bf;
{
  bf->bufPtr->changed = False;
}

/****************************************************************************
 * ETchanged - determine if buffer has changed since last write
 ***************************************************************************/

Boolean ETchanged(bf)
  BufInfo *bf;
{
  return(bf->bufPtr->changed);
}

/* ETtext: ********************************************************************
 * get line of text
 *****************************************************************************/

char *ETtext(bf, line)
     BufInfo *bf;
     int line; 					/* phys. line number */
{
  register Buffer *b = bf->bufPtr;

  if (line < 0) {
    ESmsg(bf, "ETtext - bad line", False);
    return(NULL);
  }

  if (line >= b->numlines)
    return(NULL);

  if(b->text[line] == NULL) {
    ESmsg(bf, "ETtext - NULL line", False);
    return(NULL);
  }

  return(b->text[line]);
}

/* ETgetchar: ****************************************************************
 * get character of a specified position in text
 *****************************************************************************/

char ETgetchar(bf, line, pos)
     BufInfo *bf;
     int line; 					/* phys. line number */
     int pos;                                   /* position in phys. line */
{
  register Buffer *b = bf->bufPtr;

  if (line < 0)
    fatal("ETtext - bad line", "ETtext", NULL);
  if (line >= b->numlines)
    return(0);
  if(b->text[line] == NULL)
    fatal("ETtext - NULL line", "ETtext", NULL);
  return(b->text[line][pos]);
}

/* ETlen: ********************************************************************
 * get length of line
 *****************************************************************************/

int ETlen(bf, line)
     BufInfo *bf;
     int line; 					/* phys. line number */
{
  register Buffer *b = bf->bufPtr;

  if (line < 0 || line >= b->numlines)
    fatal("ETlen - bad line", "ETlen", NULL);
  if (b->text[line] == NULL)
    fatal("ETlen - NULL line", "ETlen", NULL);
  return(strlen(b->text[line]));
}

/* ESlen *********************************************************************
 * get length of visible line (including tab length)
 ****************************************************************************/

int ESlen(bf, line)
     BufInfo *bf;
     int line; 					/* line number */
{
  register Buffer *b = bf->bufPtr;

  if (line < 0 || line >= b->numlines)
    fatal("ESlen - bad line", "ESlen", NULL);
  if (b->text[line] == NULL)
    fatal("ESlen - NULL line", "ESlen", NULL);

  return(strlen((char *) EMtext(bf, line, False)));
}

/* ETnumlines: ***************************************************************
 * get number of lines in buffer
 ****************************************************************************/

int ETnumlines(bf)
     BufInfo *bf;
{
  return(bf->bufPtr->numlines);
}

/* ETemptybuf: ****************************************************************
 * returns TRUE if text buffer is empty
 *****************************************************************************/

Boolean ETemptybuf(bf) 
     BufInfo *bf;
{
  return((Boolean)((ETnumlines(bf) == 1)&&(bf->bufPtr->text[0] == blankline)));
}

/* ETinschar: ****************************************************************
 * insert character into line
 *****************************************************************************/

void ETinschar(bf, line, pos, ch)
     BufInfo *bf;
     int line; 					/* line number */
     int pos; 					/* pos to insert at */
     int ch; 					/* character to insert */
{
  register Buffer *b = bf->bufPtr;
  int len;
  
  if (pos < 0)				/* illegal pos, exit */
    fatal("ETinschar - bad col", "ETinschar", NULL);

  if (pos > MAXLINE) {			/* line too long, ignore char */
    Bell();
    return;
  }

  len = ETlen(bf, line);
 
  if (!growline(bf, line, len + 1))
    return;

  bcopy(&b->text[line][pos], &b->text[line][pos + 1], len - pos + 1);
  b->text[line][pos] = (char)ch;
  b->changed = True;
}

/* ETinsstr: ******************************************************************
 * insert string into line, no embedded <nl> 
 *****************************************************************************/

Boolean ETinsstr(bf, line, pos, str)
     BufInfo *bf;
     int line; 					/* line number */
     int pos; 					/* pos to insert at */
     char *str;                                 /* string to insert */
{
  register Buffer *b = bf->bufPtr;
  int len, slen;
  register i;
  
  if (pos < 0 || pos > MAXLINE)    
    fatal("ETinsstr - bad pos", "ETinsstr", NULL);

  if (strchr(str, '\n'))			/* check for newline */ 
    return(False);				

  len = ETlen(bf, line);
  slen = strlen(str);
  if (!growline(bf, line, len + slen))
    return(False);

  if (pos < len)
    bcopy(&b->text[line][pos], &b->text[line][pos + slen], len - pos + slen);

  for (i = 0; i < slen; i++) 
    b->text[line][pos + i] = *str++;
  if (pos == len)
    b->text[line][pos + i] = EOS;

  b->changed = True;
  return(True);
}

/* ETdelchar: *****************************************************************
 * delete character from line
 *****************************************************************************/

Boolean ETdelchar(bf, line, col)
     BufInfo *bf;
     int line; 					/* line number */
     int col; 					/* clumn to delete */
{
  register Buffer *b = bf->bufPtr;
  int len;
  char s[2];

  if ((len = ETlen(bf, line)) <= 0)
    return(False);

  sprintf(s, "%c", b->text[line][col]);
  setKillBuffer(bf, strCopy(s));

  bcopy(&b->text[line][col + 1], &b->text[line][col], len - col + 1);
  b->changed = True;
  return(True);
}

/* ETinsline: *****************************************************************
 * insert fresh line into buffer memory before 'line'
 *****************************************************************************/

Boolean ETinsline(bf, line)
     BufInfo *bf;			/* Editor buffer */
     int *line;				/* line number */
{
  register Buffer *b = bf->bufPtr;

  if (*line < 0 || *line > b->numlines)
    fatal("ETinsline - bad line", "ETinsline", NULL);
  if (b->numlines >= b->bufferLines) {
    if (bf->flags & ED_AUTOKILL) {
      int kill = KILLAMOUNT;
      while (b->numlines >= b->bufferLines) {
	autodeleteLines(bf, kill);
	*line = MAX(*line - kill, 0);
      }
    } else {
      ESmsg(bf, "Too many lines !", True);
      return(False);
    }
  }
  if (*line < b->numlines)
    bcopy((char *)&b->text[*line], (char *)&b->text[*line + 1],
	   (b->numlines - *line) * sizeof(b->text[0]));
  if (!store(b, *line, "")) {
    ESmsg(bf, "Cannot insert line: no memory !", True);
    return(False);
  }
  b->numlines++;
  return(True);
}

/* ETdelline: *****************************************************************
 * delete line of text from buffer memory
 *****************************************************************************/

void ETdelline(bf, line)
     BufInfo *bf;
     int line;
{
  setKillBuffer(bf, strCopy(bf->bufPtr->text[line]));

  ETdellines(bf, line, 1);
}

/* ETdellines: **************************************************************
 * delete amount lines starting at line 
 ***************************************************************************/

void ETdellines(bf, line, amount)
     BufInfo *bf;
     int line, amount;
{
  register Buffer *b = bf->bufPtr;
  int cnt, i;

  if (line < 0)
    fatal("ETdellines - bad line", "ETdellines", NULL);

  if (amount <= 0)
    return;
    
  cnt = (line + amount < b->numlines) ? amount : b->numlines - line;

  for (i = 0; i < cnt; i++)		/* delete lines in buffer */
    freeline(b, line + i);

  /* move remaining lines up in buffer */
  bcopy((char *)&b->text[line + cnt], (char *)&b->text[line],
	 (b->numlines - line - cnt) * sizeof(b->text[0]));
  bzero((char *)&b->text[b->numlines - cnt], cnt * sizeof(b->text[0]));
  b->numlines -= cnt;			/* adjust number of lines in buffer */
  if (b->numlines == 0)			/* empty buffer */
    (void)ETappend(bf, "");
  b->changed = True;
}

/* ETappend: ******************************************************************
 * append new line of text to end of buffer
 *****************************************************************************/

Boolean ETappend(bf, text)
     BufInfo *bf;
     char *text; 					/* text */
{
  register Buffer *b = bf->bufPtr;

  if (b->numlines >= b->bufferLines) {
    if (bf->flags & ED_AUTOKILL)
      autodeleteLines(bf, KILLAMOUNT);
    else {
      ESmsg(bf, "Too many lines !", True);
      return(False);
    }
  }
  if(!store(b, b->numlines, text)) {
    ESmsg(bf, "Cannot append text: no memory !", True);
    return(False);
  }

  b->numlines++;
  return(True);
}

/* ETreplace: ****************************************************************
 * replace line of text with new text
 *****************************************************************************/

void ETreplace(bf, line, text)
     BufInfo *bf;
     int line; 
     char *text; 
{
  register Buffer *b = bf->bufPtr;

  if (line < 0 || line >= b->numlines)
    fatal("ETreplace - bad line", "ETreplace", NULL);

  freeline(b, line);
  if (! store(b, line, text)) 
    ESmsg(bf, "Cannot replace text: no memory", True);
}

/* locateCurrentWord: ********************************************************
 * locate the word under the cursor or befor the cursor and set two pointers:
 * lpos points to the first character of the word, rpos points to the last 
 * character of the word in corresponding line in the editor buffer. False is 
 * returned if no word can be located.
 ****************************************************************************/

Boolean locateCurrentWord(editor, lpos, rpos)
     Editor editor;
     char **lpos, **rpos;
{
  register BufInfo *bf = bufinfoptr(editor);
  register char *t = ETtext(bf, bf->curline);
  register char *llpos, *rrpos;
  int pos;
  
  if (! t) 					/* no line */
    return(False);

  if (! *t) {					/* empty line */
    *lpos = *rpos = t;
    return(True);
  }
    
  if (charInSet(t[bf->curpos], DELIMS)) {	/* cursor not inside word */
    for (pos = bf->curpos - 1; pos >= 0; pos--)	/* try to reach a word */
      if (! charInSet(t[pos], DELIMS))		/* tail of a word reached */
	break;
    if (pos < 0)
      return(False);				/* no word left to cursor */
    
    rrpos = &t[pos];				/* just beyond word */
  } else {					/* cursor in word */
    for (pos = bf->curpos + 1; pos < strlen(t); pos++)  /* locate tail */
      if (charInSet(t[pos], DELIMS)) {
	rrpos = &t[pos - 1];
	break;
      }
    
    if (pos >= strlen(t))
      rrpos = t + strlen(t) - 1;
  }
  
  llpos =  (rrpos > t) ? rrpos - 1 : t;
  
  for ( ;llpos > t; llpos--)			/* locate beg. of word */
    if (charInSet(*llpos, DELIMS)) {
      llpos++;					/* first char of word */
      break;
    }
  
  *lpos = llpos;
  *rpos = rrpos;
  return(True);
}

/* autodeleteLines: **********************************************************
 * on buffer-overflow delete a certain amount amount of line at the top of the
 * buffer 
 *****************************************************************************/

void autodeleteLines(bf, amount)
     BufInfo *bf;
     int amount;
{
  char msg[50];

  if (amount >= ETnumlines(bf)) {
    killbuf(bf, False);
    return;
  }

  ETdellines(bf, 0, amount);		/* delete lines in text buffer */
  bf->topline -= amount;		/* adjust topline */
  if (bf->topline < 0)
    bf->topline = 0;

  if (bf->flags & ED_MAPPED && bf->topline <= amount) {	/* in visible area */
    ESshowcur(bf, False, bf->curcol, bf->curpos);
    bf->currow = MAX(0, bf->currow - amount);
    setpos(bf);
    csrChrForget(bf);
    ESshowall(bf);
    ESsetcur(bf);
  }
  sprintf(msg, "autodelete: %d lines deleted.", amount);
  ESmsg(bf, msg, False);
}
