/************************************************************************
 *                                                                      *
 *    Copyright (c) 1991 Wilhelm Valder, Regine Freitag, Wolfgang Dzida *
 *                        All rights reserved                           *
 *                                                                      *
 *                                                                      *
 *    Permission to copy this program or any part of this program is    *
 *    granted in case of personal, non-commercial use only. Any use     *
 *    for profit or other commercial gain without written permissions   *
 *    of the authors is prohibited.                                     *
 *                                                                      *
 *    You may freely distribute this program to friends, colleagues,    *
 *    etc.  As a courtesy we would appreciate it if you left our names  *
 *    on the code as the authors.                                       *
 *    We also, of course, disavow any liabilities                       *
 *    for any bizarre things you may choose to do with the system.      *
 *                                                                      *
 *    We would appreciate though your letting us know about any         *
 *    extensions, modifications, project suggestions, etc.              *
 *    Please mail all bugs, comments, complaints and suggestions to:    *
 *                                                                      *
 *                                                                      *
 *    GMD (German National Research Center for Computer Science)        *
 *    Institut fuer Systemtechnik (F2G2)                                *
 *    P.O. Box 1240                                                     *
 *    Schloss Birlinghoven                                              *
 *    D-5205 St. Augustin 1, West Germany                               *
 *                                                                      *
 *    email:    dzida@kmx.gmd.dbp.de                                    *
 *              freitag@gmdzi.uunet                                     *
 *                                                                      *
 ************************************************************************/
#include <stdio.h>
#include <exBasics.h>
#include <exEdit.h>
#include <exLayout.h>
#include <exStxt.h>
#include <exMenu.h>
#include "exMacros.h"
#include <exErrors.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <exThermometer.h>

static char SccsID[] = "@(#)macros.c	1.2\t11/13/92";            /* sccs keyword */

char *rindex();

static char msg[100];
static char fullName[256];
static char *macroDir = (char *) 0;	/* value of the env. var. */
static _macroInit = 0;			/* set if macros initialised */

/* menu that holds the available macro sets; the full pathname of the 
   selected macro set is returned
*/

static MHmenu *macroSetMenu =(MHmenu *) 0;

static long loadTime = 0L;		/* time of last macroTable load */
static Boolean rebuildMenus = True;	/* true if menus need rebuild */

static MHmenu *makeMacroSetList();
static MHmenu *makeMacroList();
static getMacroText();
static loadMacroNames();
static removeMacroDir();
static storeMacroText();

static struct {
  char *macroSet; 			/* full path of macro set directory */
  char *macros[MDS_MAXITEM];		/* list of macros */
} macroTable[MDS_MAXMENU];
     
static struct {				/* associates menu-structures */
  MHmenu *mptr;				/* with menus for loaded macro sets*/
  exMenu *menu;
} macroMenuTable[MDS_MAXMENU];
   
/* storeMacroText: **********************************************************
 * stores the <text> in a file named <macro> under the directory <menu>. The 
 * root directory of the macro-defintions is determined by the 
 * environment-variable '$MACROPATH'
 ****************************************************************************/
     
static storeMacroText(w, macroSet, macro, text)
     Widget w;
     char *macroSet;    /* the macro directory (full path) */
     char *macro;   	/* the macro name */
     char *text;       	/* the macro body */
{
  FILE  *fd;
  int len, i = 0;
  
  if (! macroSet || ! macro || ! text || ! *macro || ! *macroSet) {
    setError(ERR_NULL);
    return(-1);
  }
  
  if (! fileExists(macroSet)) {		/* subdirectory does not exist */
    if (mkdir(macroSet, 0755) < 0) {	/* create it */
      setError(ERR_CREATE);
      return(-1);
    }
  }
  
  /* generate the full name of the macro (without suffix) */
  if (makeFullName(macroSet, macro, fullName, sizeof(fullName)) < 0) {
    setError(ERR_ALLOC);
    return(-1);
  }

  /* add suffix to full macro name */
  if (strlen(fullName) + 1 + strlen(MACRO_SUFFIX) > sizeof(fullName)) {
    setError(ERR_ALLOC);
    return(-1);
  }

  strcat(fullName, MACRO_SUFFIX);

  if ((fd = fopen(fullName, "w")) == (FILE *) 0) {
    setError(ERR_WRITE);
    return(-1);
  }
  
  sprintf(msg, "Storing macro -- %s -- into macro set -- %s --", 
	  						macro, macroSet);
  
  initThermometer(XtDisplay(w), w, 0, 0, strlen(msg), msg, NULL);
  len = strlen(text);
  
  if (len < 50) {		 	/* do not show progress */
    setThermometer(100, 100, 100);
    while (*text)
      fputc(*text++, fd);
  } else {
    while (*text) {
      if (len > 10 && ! (i++ % (len / 10)))
	setThermometer(100, 100, i * 100 / (len - 1));
      fputc(*text++, fd);
    }
  }
  
  hideThermometer(1);
  fclose(fd);
  setError(ERR_NOERR);
  return(0);
}

/* removeMacroFile: ***********************************************************
 * delete the macro with the given name by removing the corresponding file 
 * from the directory <menu>. 
 *****************************************************************************/

removeMacroFile(w, file)
     Widget w;
     char *file;	/* the macro file */
{
  char *macroSet;	/* the macroSet name */
  char *macro;		/* macro name */
  int rc;
  static char fileName[200];
  
  if (! file) {
    setError(ERR_EXISTS);
    return(-1);
  }
  
  strcpy(fileName, file);
  strcat(fileName, MACRO_SUFFIX);
  
  rc = unlink(fileName);			/* delete file */
  strcpy(fileName, file);
  if (! (macro = rindex(fileName, '/')))	/* extract macro name */
    return(rc);
  *macro++ = EOS;
  
  if (! (macroSet = rindex(fileName, '/')))	/* macro set name */
    return(rc);
  macroSet++;
  
  if (rc < 0) {
    setError(ERR_ACCESS);
    return(-1);
  } else {
    if (removeMacroDir(w, macroSet) < 0) 	/* not last macro in set */
      rc = 1;  					/* macro deleted from set */
    else					/* last macro, dir. removed */
      rc = 2; 					
  }
  
  touchFile(macroDir, time(0), time(0));

  return(rc);
}

/* removeMacroDir ************************************************************
 * remove the specified macro - directory if no macros are contained 
 ****************************************************************************/

static removeMacroDir(w, macroSet)
     Widget w;
     char *macroSet;
{
  if (! macroSet)
    return(-1);

  return(rmdir(macroSet));
}
    
/* loadMacroSet: ************************************************************
 * load the macro definitions that are stored in the directory <menu> and 
 * build the corresponding menu structure; then insert the new menu into the 
 * specified menubar;
 ****************************************************************************/

loadMacroSet(menubar, macroSet)
     exMenubar *menubar;/* insert menu there */
     char *macroSet;	/* the absolute macroset name path (sub-directory) */
{
  register char *exdot;
  register MHitem *iptr;
  register MHiReq *ireq;
  MHmenu *mptr;
  int i, cnt, indx;
  char *table[MDS_MAXITEM];

  setWorkApplication();

  if ((cnt = loadDirectory(macroSet, table, MDS_MAXITEM)) < 0) {
    setError(ERR_OPEN);
    return(-1);
  }
  
  if ((mptr = createMenu()) == (MHmenu *) 0) {	   	/* make the menu */
    setError(ERR_CREATE);
    return(-1);
  }

  insertMenu(mptr);

  setSelCnt(mptr, 1);
  setSelType(mptr, MDS_CNT_EQ);
  initMenu(mptr, MDS_M_PULLD, getMacroSetName(macroSet));
  mptr->m_hlp->hlpmsg = strCopy("Please select one of the macros.");
  mptr->m_key = EOS;
  
  for (i = 0; i < cnt; i++) {			/* make the items */
    if ((exdot = rindex(table[i], '.')) == (char *) 0)
      continue;
    
    if (strcmp(exdot, MACRO_SUFFIX) == 0) {	/* found a macro file */
      *exdot = EOS;
      
      if (makeFullName(macroSet, table[i], fullName, sizeof(fullName)) < 0)
	continue;
      
      iptr = newItem(mptr, table[i], EOS);	/* create item */
      iptr->i_return->retPattern = strCopy("$");
      
      ireq = newRequest();
      setRequestData(ireq, fullName);
      setReqFnct(ireq, getMacroText);
      insertRequest(iptr, ireq, 0);
    }
    checkRequests(iptr);
    free(table[i]);
  }

  /* store macro set information */

  for (i = 0, indx = 0; i < MDS_MAXMENU; i++) {
    if (! macroMenuTable[i].mptr) {
      if (! indx)
	indx = i;				      /* 1st free entry */
    }
  }

  macroMenuTable[indx].mptr = mptr;
  macroMenuTable[indx].menu = exMakeMenu(NULL, menubar, (Widget) NULL, mptr);
  
  setError(ERR_NOERR);
  return(0);
}

/* hideMacroSet: ***********************************************************
 * remove the macro-menu with the given name from the menubar
 *****************************************************************************/

hideMacroSet(macroSet)
     char *macroSet;	/* macroset name (subdirectory rel. to $MACROPATH */
{
  register MHmenu *mptr;
  register i;

  setWorkApplication();

  for (i = 0; i < MDS_MAXMENU; i++) {
    if (mptr = macroMenuTable[i].mptr) {		
      if (strcmp(mptr->m_repr->name, macroSet) == 0) {	/* found it */
	deleteWorkMenu(mptr);				/* free structure */
	XtDestroyWidget(macroMenuTable[i].menu->button);/* delete in menubar */
	macroMenuTable[i].mptr = (MHmenu *) 0;
	macroMenuTable[i].menu = (exMenu *) 0;
	return(0);
      }
    }
  }
  setError(ERR_NOEXIST);
  return(-1);
}

/* loadMacroText: ************************************************************
 * loads the macro text of the specified makro into the macro Editor
 ****************************************************************************/

int loadMacroText(macro, buffer)
     char *macro;
     char **buffer;
{
  int nb;
  RequestParm localParm;

  if (! macro) {
    setError(ERR_NULL);
    return(-1);
  }

  localParm.data = macro;

  if ((nb = getMacroText(&localParm)) < 0)
    return(-1);

  *buffer = strCopy(localParm.buffer);
  return(nb);
}
 
/* getMacroText: *************************************************************
 * this function stores the text of the macro of the requestParm-structure. 
 * The absolute name of the file containing the macro text is provided in the
 * request data.
 ****************************************************************************/

static getMacroText(parms)
     RequestParm *parms;
{
  FILE *fd;
  register c, i = 0;
  char *name;
  int size;
  
  name = Malloc(strlen(parms->data) + strlen(MACRO_SUFFIX) + 1);
  strcpy(name, parms->data);
  strcat(name, MACRO_SUFFIX);

  if (! fileExists(name)) {			/* check if file exists */
    setError(ERR_NOEXIST);
    return(-1);
  }
  
  if ((size = getFileSize(name)) < 0) {		/* get file size */
    setError(ERR_ACCESS);
    return(-1);
  }

  if ((fd = fopen(name, "r")) == (FILE *) 0) {	/* open file */
    setError(ERR_OPEN);
    return(-1);
  }

  parms->buffer = Malloc(size + 1);
  
  while ((c = getc(fd)) != EOF)
    parms->buffer[i++] = (char) c;
  
  fclose(fd);
  free(name);
  return(size);
}

/* selectMacro: ***************************************************************
 * show the user a menu of macros in the given macros set and let him select. 
 * The user may select one macro (multiple = False) or more than one macro 
 * (multiple = True). The absolute path of the macro-file is returned.
 *****************************************************************************/

char *selectMacro(w, macroSet, multiple)
     Widget w;
     char *macroSet;		/* full qualified path-name */
     Boolean multiple;		/* true, if multi-selection */
{
  MHmenu *mptr;
  exMenu *macroList;
  char *ptr;
  int x,y;

  if (! macroSet)			/* no macro set specified */
    return(NULL);

  getMouse_xy(XtDisplay(w), &x, &y);

  if ((mptr = makeMacroList(w, macroSet, multiple)) == (MHmenu *) NULL) {
    setError(ERR_ALLOC);
    return(NULL);
  }

  setMenuPos(mptr, x, y, MDS_POS_ABS);
  
  if (! (macroList = exMakeMenu(NULL, NULL, w, mptr))) {
    setError(ERR_CREATE);
    return((char *) 0);
  }

  exCallMenu(macroList);

  ptr = strCopy(exGetMenuReturnValue(macroList));
  
  freeMenu(macroList);

  return(ptr);
}

/* selectMacroSet: ***********************************************************
 * shows the user a menu of macro sets and lets him select a set. The 
 * absolute path of the macro-set directory is returned.
 *****************************************************************************/

char *selectMacroSet(w)
     Widget w;
{
  MHmenu *mptr;
  exMenu *macroSetList;
  char *ptr;
  int x,y;

  getMouse_xy(XtDisplay(w), &x, &y);

  if (! (mptr = makeMacroSetList(w)))
    return((char *)0);

  setMenuPos(mptr, x, y, MDS_POS_ABS);
  
  if (! (macroSetList = exMakeMenu(NULL, NULL, w, mptr)))
    return((char *) 0);

  exCallMenu(macroSetList);

  ptr = strCopy(exGetMenuReturnValue(macroSetList));
  
  freeMenu(macroSetList);

  return(ptr);
}

/* getNextMacroSet: **********************************************************
 * search for the next or previous macro set;
 * absolute path of the macro-set directory is returned.
 *****************************************************************************/

char *getNextMacroSet(macroSet)
     char *macroSet;
{
  register i;

  for (i = 0; i < MDS_MAXMENU && macroTable[i].macroSet; i++)
    if (strcmp(macroSet, macroTable[i].macroSet) == 0) {
      if (macroTable[i + 1].macroSet)               /* take next macro set */
	return(macroTable[i + 1].macroSet);
      else if (i > 0 && macroTable[i - 1].macroSet)  
        return(macroTable[i - 1].macroSet);  /* take previous macro set */
      else                                 
	return(NULL);                 /* no other macro set available */
    }
  return(NULL);
}

/* storeMacro: ****************************************************************
 * store the contents of the buffer of the macro editor as a new macro into 
 * the specified file.
 *****************************************************************************/

storeMacro(w, macroSet, macro, txt)
     Widget w;
     char *macroSet;	    	/* the menu name where to store */
     char *macro;		/* the name of the macro */
     char *txt;
{
  int rc;

  if ((rc = storeMacroText(w, macroSet, macro, txt)) != -1)
    touchFile(macroDir, time(0), time(0));
  return(rc);
}

/* selectDisplayedMacros: *****************************************************
 * create a menu that holds the displayed macro menus and let the user select.
 * Provide the selected names to the calling function.
 *****************************************************************************/

char *selectDisplayedMacros(w)
     Widget w;
{
  register i;
  register MHmenu *mptr;
  register MHitem *iptr;
  int x, y;
  exMenu *macroList;
  char *ptr;

  getMouse_xy(XtDisplay(w), &x, &y);

  setWorkApplication();

  if ((mptr = createMenu()) == (MHmenu *) 0) {		/* create a new menu */
    setError(ERR_CREATE);
    return((char *) 0);
  }
  
  initMenu(mptr, MDS_L_POPUP, "loaded macros");
  insertMenu(mptr);
  setSelCnt(mptr, 1);
  setSelType(mptr, MDS_CNT_EQ);
  setMenuPos(mptr, x, y, MDS_POS_ABS);
  mptr->m_hlp->hlpmsg = strCopy("Please select one of the macros !");

  for (i = 0; i < MDS_MAXMENU; i++)
    if (macroMenuTable[i].mptr) {
      iptr = newItem(mptr, macroMenuTable[i].mptr->m_repr->name, EOS);
      iptr->i_return->retPattern = 
	         strCopy(macroMenuTable[i].mptr->m_repr->name);
    }
  
  if (! (macroList = exMakeMenu(NULL, NULL, w, mptr)))
    return((char *) 0);

  exCallMenu(macroList);
  ptr = strCopy(exGetMenuReturnValue(macroList));
  freeMenu(macroList);

  return(ptr);
}

/* exInitMacro: **************************************************************
 * inititialise some variables used when handling with macros:
 * the macro directory: 				--> macroDir
 * the list of macroSets and corresponding macros 	--> macroTable
 *****************************************************************************/

Boolean exInitMacro(w)
     Widget w;
{
  register i, j;

  if (_macroInit)
    return(True);

  for (i = 0; i < MDS_MAXMENU; i++) {		/* clear macro table */
    for (j = 0; j < MDS_MAXITEM; j++)
      macroTable[i].macros[j] = NULL;
    macroTable[i].macroSet = NULL;
  }
    
  if ((macroDir = strCopy(getenv("MACROPATH"))) == (char *) 0) {
    if ((macroDir = strCopy(getenv("HOME"))) == (char *) 0) {
      setError(ERR_NOEXIST);
      exAlert(w, -1, -1, "MacroEditor: Cannot locate your macro directory: \nPLease set the Environment variable MACROPATH.");
      exit(1);
    }
  }
  
  if (! makeMacroSetList(w)) 
    return(False);
  
  _macroInit++;
  return(True);
}

/* loadMacroTables: ***********************************************************
 * loads the users macro-set names into the macroTable 'macroSetTable. 
 * Additionally, for each macro set the names of the corresponding macros 
 * are loaded into the table 'macroTable'. Loading is only performed, if 
 * the directories has changed since the last loading.
 ****************************************************************************/

loadMacroTables()
{
  int macroSetCnt;
  struct stat sbuf;
  register i, cnt = 0;
  char *macroSetTable[MDS_MAXMENU];
  
  rebuildMenus = False;
  
  if (stat(macroDir, &sbuf) < 0) 		/* cannot stat, so reload */
    rebuildMenus = True;
  else if (sbuf.st_ctime > loadTime) 		/* directory changed */
    rebuildMenus = True;
  
  if (! rebuildMenus)
    return(0);
  
  /* exError("ME", "(Re-)loading macro-tables !", NULL);*/
  
  /* load the names of the sub-directories of the user's macro directory */
  if ((macroSetCnt = loadDirectory(macroDir, macroSetTable,MDS_MAXMENU)) < 0) {
    setError(ERR_OPEN);
    return(-1);
  }
  
  /* for each macro-set, load the corresponding macros into the macroTable */
  for (i = 0; i < macroSetCnt; i++) {
    if (makeFullName(macroDir,macroSetTable[i],fullName,sizeof(fullName))< 0) {
      free(macroSetTable[i]);
      continue;
    }
    
    macroTable[cnt].macroSet = strCopy(fullName);
    loadMacroNames(fullName, macroTable[cnt].macros, MDS_MAXITEM); 
    free(macroSetTable[i]);
    cnt++;
  }
  
  for (; i < MDS_MAXMENU; i++)		/* clear unused part of table */
    macroTable[i].macroSet = (char *) 0;
  
  loadTime = sbuf.st_ctime;
  return(0);
}

/* loadMacroNames: ************************************************************
 * load the nams of the files that contains macros into the specified table
 *****************************************************************************/

static loadMacroNames(mDir, table, size)
     char *mDir;
     char *table[];
     int size;
{
  int cnt;
  char *MACROtable[MDS_MAXITEM];
  register char *exdot;
  register i, cnt0;
  
  if ((cnt = loadDirectory(mDir, MACROtable, MDS_MAXITEM)) < 0) {
    setError(ERR_OPEN);
    return(-1);
  }
  
  for (i = 0, cnt0 = 0; i < cnt && cnt0 < size; i++) {
    if ((exdot = rindex(MACROtable[i], '.')) == (char *) 0)
      continue;
    
    if (strcmp(exdot, MACRO_SUFFIX) == 0) {	/* found a macro file */
      *exdot = EOS;
      free(table[cnt0]);
      table[cnt0] = strCopy(MACROtable[i]);
      free(MACROtable[i]);
      cnt0++;
    }
  }
  
  bzero(&table[cnt0], (size - i) * sizeof(char *));
  
  return(0);
}

/* makeMacroList: ************************************************************
 * make a list of macros in the given macro set and let the user select.
 * Depending on the parameter 'multiple', one macro (multiple == False) or
 * more than one macro (multiple == True) may be selected. The absolute 
 * path-name of the file containing the macro is returned.
 *****************************************************************************/

static MHmenu *makeMacroList(w, macroSet, multiple)
     Widget w;
     char *macroSet;		/* current macro set (full path name)*/
     Boolean multiple;		/* allow selection of multiple macros */
{
  register i;
  register MHitem *iptr;
  register MHmenu *mptr;
  char **macros = NULL;

  /* search for macro set in the table of currently known sets */

  loadMacroTables();

  for (i = 0; i < MDS_MAXMENU && macroTable[i].macroSet; i++) {
    if (strcmp(macroTable[i].macroSet, macroSet) == 0) {
      macros = macroTable[i].macros;
      break;
    }
  }
  
  if (! macros)					/* macro set not found */
    return((MHmenu *) 0);

  setWorkApplication();				/* application for menus */
  
  if ((mptr = createMenu()) == (MHmenu *) 0) {	/* create a new menu */
    setError(ERR_CREATE);
    return((MHmenu *) 0);
  }
  
  initMenu(mptr, MDS_L_POPUP, "macros");
  insertMenu(mptr);
  setSelCnt(mptr, 1);
  setSelType(mptr, multiple ? MDS_CNT_GE : MDS_CNT_EQ);
  setMenuPos(mptr, 0, 0, MDS_POS_ABS);
  mptr->m_hlp->hlpmsg = strCopy("Please select macros !");

  /* create an item for each macro */
  for (i = 0; i < MDS_MAXITEM && macros[i]; i++) {
    if (makeFullName(macroSet, macros[i], fullName, sizeof(fullName)) < 0)
      continue;
    
    iptr = newItem(mptr, macros[i], EOS);
    iptr->i_return->retPattern = strCopy(fullName);
  }
  
  return(mptr);
}

/* makeMacroSetList: **********************************************************
 * make a list of macro-sets currently in the macrotable. The menu is 
 * created only if the corresponding directories have changed. 
 *****************************************************************************/

static MHmenu *makeMacroSetList(w)
     Widget w;
{
  static char *therm_txt = "Looking up macro sets .... ";
  register i;
  register MHitem *iptr;
  MHmenu *mptr;
  
  setWorkApplication();				/* application for menus */
  loadMacroTables();
  
  if (! rebuildMenus) 
    if (macroSetMenu)
      return(macroSetMenu); 
  
  if (macroSetMenu) {				/* delete old menus */
    for (i = 0; i < macroSetMenu->itemCnt; i++) {
      if ((mptr = getMenuDescrByName(macroSetMenu->i_table[i]->i_repr->name)) 
	  && (!macroSetIsLoaded(mptr->m_repr->name)))
	deleteWorkMenu(mptr);
    }                       /* menus of loaded macro sets are not deleted */
    deleteWorkMenu(macroSetMenu);
  }
  
  if ((mptr = createMenu()) == (MHmenu *) 0) {	/* create a new menu */
    setError(ERR_CREATE);
    return((MHmenu *) 0);
  }
  
  initMenu(mptr, MDS_M_POPUP, "macro sets");
  insertMenu(mptr);
  setSelCnt(mptr, 1);
  setSelType(mptr, MDS_CNT_EQ);
  setMenuPos(mptr, 0, 0, MDS_POS_ABS);
  mptr->m_hlp->hlpmsg = strCopy("Please select one of the macro sets !");
  
  initThermometer(XtDisplay(w), w, 0, 0, strlen(therm_txt), therm_txt, NULL);

  for (i = 0; i < MDS_MAXMENU; i++) {
    
    setThermometer(100, 100, i * 100 / (MDS_MAXMENU - 1));
    
    if (! macroTable[i].macroSet)
      continue;
    
    /* create the corresponding item */
    iptr = newItem(mptr, getMacroSetName(macroTable[i].macroSet), EOS);
    iptr->i_return->retPattern = strCopy(macroTable[i].macroSet);
  }
  
  hideThermometer(10);

  return(macroSetMenu = mptr);
}

/* removeMacroSet: ************************************************************
 * remove all macros from the given macroset, i.e. remove all files in the 
 * corresponding directory together with the directory itsself. The number 
 * of deleted macro files is returned.
 *****************************************************************************/

int removeMacroSet(macroSet)
     char *macroSet;
{
  register i, cnt;
  char *macTable[MDS_MAXMENU];
  Boolean ok = True;
  
  setError(ERR_NOERR);

  if ((cnt = loadDirectory(macroSet, macTable, MDS_MAXMENU)) < 0) {
    setError(ERR_OPEN);
    return(-1);
  }

  for (i = 0; i < cnt; i++) {
    makeFullName(macroSet, macTable[i], fullName, sizeof(fullName));

    if (unlink(fullName) < 0) {
      setError(ERR_OPEN);
      ok = False;
    }
    free(macTable[i]);
  }

  if (rmdir(macroSet) < 0) {
    setError(ERR_OPEN);
    ok = False;
  }

  if (ok)
    rebuildMenus = True;

  return(ok ? cnt : -cnt);
}

/* macroSetEdists: ***********************************************************
 * look if the specified macro set does already exist.
 ****************************************************************************/

Boolean macroSetExists(macroSet)
     char *macroSet;			/* full name of macro set */
{
  register i;

  /* search for macro set in the table of currently known sets */

  loadMacroTables();

  for (i = 0; i < MDS_MAXMENU && macroTable[i].macroSet; i++) {
    if (strcmp(macroTable[i].macroSet, macroSet) == 0) 
      return(True);
  }
  
  return(False);
}

/* makeMacroDirPath: *********************************************************
 * given a macro-set, this function concatenates it with the current 
 * macro-directory prefix to return the full-qualified path name of the given
 * macro-dir.
 ****************************************************************************/

char *makeMacroDirPath(macroSet)
     char *macroSet;
{
  register char *ptr;

  if (! macroSet || ! *macroSet)
    return(char *) 0;

  ptr = (char *) malloc(strlen(macroSet) + strlen(macroDir) + 2);

  sprintf(ptr, "%s/%s", macroDir, macroSet);

  return(ptr);
}

/* findFirstMacro: ************************************************************
 * find first macro of a given macro set
 *****************************************************************************/

char *findFirstMacro(macroSet)
     char *macroSet;		/* current macro set (full path name)*/
{
  register i, j = 0;
  char *macroName;

  /* search for macro set in the table of currently known sets */

  for (i = 0; i < MDS_MAXMENU && macroTable[i].macroSet; i++) {
    if (strcmp(macroTable[i].macroSet, macroSet) == 0) {
      while(j < MDS_MAXITEM && (!macroTable[i].macros[j]))
	j++;
      macroName = macroTable[i].macros[j];
      return(strCopy(macroName));
    }
  }
  
  return((char *) 0);
}

/* macroSetIsLoaded: *******************************************************
 * check if there exists an entry in table `macroMenuTable` corresponding to 
 * a given macro set (meaning that this macro set is already loaded into the
 * command listener)
 *****************************************************************************/

Boolean macroSetIsLoaded(macroSet)
     char *macroSet;		/* absolute macro set name path */
{
  register i;
  char *macroName;

  macroName = getMacroSetName(macroSet);

  /* search for macro set in the table of currently loaded sets */

  for (i = 1; i < MDS_MAXMENU && macroMenuTable[i].mptr; i++) {
    if (strcmp(macroName, macroMenuTable[i].mptr->m_repr->name) == 0)
      return(True);
  }
  
  return(False);
}


/* getMacroSetName: *******************************************************
 * return macro set name for a given absolute macro set pathname
 **************************************************************************/

char *getMacroSetName(macroSet)
     char *macroSet;		/* absolute macro set path */
{
  char *macroName;

  macroName = rindex(macroSet, '/');
  if (macroName)
    return(++macroName);
  else
    return(macroSet);
}
