/************************************************************************
 *                                                                      *
 *    Copyright (c) 1991 Wilhelm Valder, Regine Freitag, Wolfgang Dzida *
 *                        All rights reserved                           *
 *                                                                      *
 *                                                                      *
 *    Permission to copy this program or any part of this program is    *
 *    granted in case of personal, non-commercial use only. Any use     *
 *    for profit or other commercial gain without written permissions   *
 *    of the authors is prohibited.                                     *
 *                                                                      *
 *    You may freely distribute this program to friends, colleagues,    *
 *    etc.  As a courtesy we would appreciate it if you left our names  *
 *    on the code as the authors.                                       *
 *    We also, of course, disavow any liabilities                       *
 *    for any bizarre things you may choose to do with the system.      *
 *                                                                      *
 *    We would appreciate though your letting us know about any         *
 *    extensions, modifications, project suggestions, etc.              *
 *    Please mail all bugs, comments, complaints and suggestions to:    *
 *                                                                      *
 *                                                                      *
 *    GMD (German National Research Center for Computer Science)        *
 *    Institut fuer Systemtechnik (F2G2)                                *
 *    P.O. Box 1240                                                     *
 *    Schloss Birlinghoven                                              *
 *    D-5205 St. Augustin 1, West Germany                               *
 *                                                                      *
 *    email:    dzida@kmx.gmd.dbp.de                                    *
 *              freitag@gmdzi.uunet                                     *
 *                                                                      *
 ************************************************************************/
/****************************************************************************
 * this file contains all those functions which must be accessable to use
 * the menu system
 ****************************************************************************/

#include <sys/types.h>
#include <sys/dir.h>
#include "exMenu.h"
#include <exErrors.h>
#include <exFonts.h>

static char SccsID[] = "@(#)Minterface.c	1.2\t6/7/93";            /* sccs keyword */

static ApplicationNode *exWorkApplication = (ApplicationNode *) 0;  
exMenubar *createMenuBar();
void postPanel();

/***************************************************************************
 *
 *			PUBLIC ROUTINES
 *
 **************************************************************************/

/* exInitMenus: ************************************************************
 * initialises the menu system; creates default gc's, loads pixmaps, defines
 * cursors;
 **************************************************************************/

void exInitMenus(d)
     Display *d;
{
  _exInitMenus(d);
}

/* exMakeMenu: *************************************************************
 * this is the highlevel function that is used to create menus from a given 
 * menu description.
 ***************************************************************************/

exMenu *exMakeMenu(aptr, menubar, parent, mptr)
     ApplicationNode *aptr;
     exMenubar *menubar;
     Widget parent;
     MHmenu *mptr;
{
  exMenu *m;
  exMenu *exCreateMenuStruct();
  
  if (! (m = exCreateMenuStruct(aptr, mptr)))
    return((exMenu *) NULL);

  if (isPopup(mptr))
    exCreatePopupMenuRepr(parent, m);
  else if (isPulldown(mptr))
    exCreatePulldownMenuRepr(menubar, m);
  else
    m = (exMenu *) NULL;
  
  return(m);
}

/* exMakePanel: *************************************************************
 * this is the highlevel function that is used to create a panel from a given 
 * menu description (see exMakeMenu()). Additional parameters define the 
 * of rows and the width of each row, if they are fixed (width > 0).
 ***************************************************************************/

exMenu *exMakePanel(aptr, parent, mptr, cols, width, forceScrollbar)
     ApplicationNode *aptr;
     Widget parent;
     MHmenu *mptr;
     int cols;		/* number of cols */
     int width;		/* fixed widt of each column (if > 0) */
     Boolean forceScrollbar;
{
  exMenu *m;
  exMenu *exCreateMenuStruct();
  
  if (! (m = exCreateMenuStruct(aptr, mptr)))
    return((exMenu *) NULL);

  if (forceScrollbar)
    m->flags |= SCROLLABLE;

  exCreatePanelRepr(parent, m, cols, width);
  
  return(m);
}

/* exCreateMenuBar: **********************************************************
 * creates a menu bar to be used with pulldown menus; A function is registered 
 * that is called whenever a ProperNotify event occurs. By this mechanism 
 * Return values are provided to the application (see exButtonDownProc())
 ****************************************************************************/

exMenubar *exCreateMenuBar(parent, mb_args, mb_cnt, getReturnValue)
     Widget parent;
     Arg mb_args[];
     int mb_cnt;
     void (* getReturnValue)();
{
  return(createMenuBar(parent, mb_args, mb_cnt, getReturnValue));
}

/* exCallMenu: ***************************************************************
 * post a menu and wait until menu-selection finished
 ****************************************************************************/

Boolean exCallMenu(m)
     exMenu *m;
{
  if (! (m->flags & POPUP)) {
    setError(ERR_TYPE);
    return(False);
  }

  postMenu(m);

  setError(ERR_NOERR);
  return(True);
}

/* exHidePanel: **************************************************************
 * hide a panel 
 ****************************************************************************/

Boolean exHidePanel(m)
     exMenu *m;
{
  XtUnmapWidget(m->pane);
  return(True);
}

/* exCallPanel: ***************************************************************
 * display a panel ready for execution at the specified position
 ****************************************************************************/

Boolean exCallPanel(m, x, y)
     exMenu *m;
{
  if (! (m->flags & PANEL)) {
    setError(ERR_TYPE);
    return(False);
  }
  
  postPanel(m, x, y);
  return(True);
}

/* exGetMenuReturnValue: ****************************************************
 * retrieve the return value from a given menu and clear it 
 ****************************************************************************/

char *exGetMenuReturnValue(m)
     exMenu *m;
{
  register char *ptr = NULL;
  
  if (m->assembledReturnValue) {
    ptr = strCopy(m->assembledReturnValue);
    free(m->assembledReturnValue);
    m->assembledReturnValue = NULL;
  }

  return(ptr);
}

/* getMenuEventReturnValue: **************************************************
 * retrieves the result of a menu selection from the according property of 
 * the specified widget
 *****************************************************************************/

char *getMenuEventReturnValue(event)
     XPropertyEvent *event;
{
  Atom ret_type;
  int ret_format;
  unsigned long ret_items;
  unsigned long rem_bytes;
  unsigned char *value;
  char *ret;

  if (! checkEvent(event))
    return(NULL);

  XGetWindowProperty(event->display, event->window, event->atom, 0L, 8192L, 
		     False, XA_STRING, &ret_type, &ret_format, &ret_items, 
		     &rem_bytes, &value);
  if (value) {
    ret = strCopy(value);
    XFree((char *) value);
    setError(ERR_NOERR);
  } else {
    setError(ERR_VALUE);
    ret = NULL;
  }

  return(ret);
}
  
/* loadMenus: ****************************************************************
 * generates the runtime structures for the menus in the given application 
 * node. the handle of the generated menus (i.e. the window identifier of 
 * the associated window is placed into the array 'menus' if non-zero. The 
 * pointer 'mcnt' holds the size of the array on procedure invocation and 
 * returns the number of stored menu descriptions.
 *****************************************************************************/

Boolean loadMenus(mbar, parent, aptr, menus, mcnt)
     exMenubar *mbar;
     Widget parent;
     ApplicationNode *aptr;
     MenuDescr menus[];
     int *mcnt;
{
  register i;
  register cnt = 0;
  register MHmenu *mptr;
  register exMenu *menu;
  Boolean err = True;
  int menuCnt = (mcnt) ? *mcnt : 0;

  if (! aptr) 
    return(False);

  for (i = 0; i < aptr->menuCnt; i++) {
    if ((mptr = aptr->m_table[i]) == (MHmenu *) NULL)
      continue;
    
    if (isPopup(mptr)) {
      if (cnt >= menuCnt) {
	err = False;
	continue;
      }
      if (!(menu = exMakeMenu(aptr, NULL, parent, mptr))) {
	err = False;
      } else {
	menus[cnt].menu = menu;
	menus[cnt].name = strCopy(mptr->m_repr->name);
	menus[cnt].type = MDS_M_POPUP;
	cnt++;
      }
    } else if (isPulldown(mptr)) {
      if (cnt >= menuCnt)
	exMakeMenu(aptr, mbar, NULL, mptr);
      else {
	if (!(menu = exMakeMenu(aptr, mbar, NULL, mptr)))
	  err = False;
	else {
	  menus[cnt].menu = menu;
	  menus[cnt].name = strCopy(mptr->m_repr->name);
	  menus[cnt].type = MDS_M_PULLD;
	  cnt++;
	}
      }
    } else
	err = False;
  }
  
  if (mcnt) 
    *mcnt = cnt;
  
  return(err);
}

/* loadCurrentApplication ***************************************************
 * creates descriptions for the menus contained in the current application
 ***************************************************************************/

Boolean loadCurrentApplication(mbar, parent, menus, mcnt)
     exMenubar *mbar;
     Widget parent;
     MenuDescr menus[];
     int *mcnt;
{
  return(loadMenus(mbar, parent, currentApplication(), menus, mcnt));
}

/* initMenus *****************************************************************
 * get the user applications (stored in path specified by environment variable
 * MENUPATH) and insert them into the MDSarray
 ****************************************************************************/

Boolean initMenus(mbar, parent, file, menus, mcnt)
     exMenubar *mbar;	/* the menubar window for pulldown menus */
     Widget parent;         
     char *file;	/* file that holds the MDS-description */
     MenuDescr menus[]; /* holds the generated popupmenus */
     int *mcnt;		/* initially holds the size of popupMenus, 
			   receives the number of generated popupMenus */
{ 
  char *getenv();
  char fullName[256];
  char *menupath;
  
  if (MDS[0] == (ApplicationNode *) 0) 		/* initialize MDS array */
    initMDS();

  if (*file == '/') {				/* absolute file name */
    strcpy(fullName, file);
  } else {					/* create file name */
    if (! (menupath = getenv("MENUPATH"))) {
      setError(ERR_NOEXIST);
      return(False);
    }    

    if (makeFullName(menupath, file, fullName, 256) < 0) {
      setError(ERR_MAX);
      return(False);
    }
  }

  if (restoreFromFile(parent, fullName, 0) != True)
    return(False);

  return(loadCurrentApplication(mbar, parent, menus, mcnt));
}

/* getMenuByName **********************************************************
 * look up the runtime structure for a menu with the given name
 ****************************************************************************/

exMenu *getMenuByName(name, menusArray, menuNo)
     char *name;
     MenuDescr menusArray[];
     int menuNo;
{
  register i;

  if (name == (char *) 0 || menuNo <= 0) {
    setError(ERR_NULL);
    return((exMenu *) NULL);
  }

  for (i = 0; i < menuNo; i++)
    if (menusArray[i].name && (strcmp(menusArray[i].name, name) == 0)) {
      setError(ERR_NOERR);
      return (menusArray[i].menu);
    }

  setError(ERR_NOEXIST);
  return((exMenu *) 0);
}

/* setInitialMask ************************************************************
 * assign new initial activity mask to menu which is used if no item is 
 * selected when menu appears 
 ****************************************************************************/

setInitialMask(m, am)
     exMenu *m;
     int am[];        /* array of length 'menu->itemCnt' which has a '0'
			 for each non-selectable item and a '1' for 
			 each selectable item */
{
  register i;

  for (i = 0; i < m->itemCnt; i++)
    if (am[i])
      AM_SET(&m->initialMask, i); 
    else
      AM_CLR(&m->initialMask, i); 

  checkMenuActivityMask(m, False);     /* adjust current activity mask */
}

/* menuBarHeight ************************************************************
 * calculate the height of the menubar 
 ****************************************************************************/

menuBarHeight(mb)
     exMenubar *mb;
{
  Arg args[20];
  int cnt, height;
  Dimension mh, bw, st, mbot, mtop;
  XmFontList fl;
  XmString xstr;

  Dimension dbst, sad, hlt;
  if (! mb) 
    return(-1);

  /* retrieve values from the menubar's rowcolumn widget */
  
  cnt = 0;
  mh = bw = st = 0;
  XtSetArg(args[cnt], XmNshadowThickness, &st); cnt++;
  XtSetArg(args[cnt], XmNmarginHeight, &mh); cnt++;
  XtSetArg(args[cnt], XmNborderWidth, &bw); cnt++;
  XtGetValues(mb->menubar, args, cnt);

  height = 2 * (mh + bw + st);

  /* retrieve values from button if any */
  cnt = 0;
  mh = bw = st = mbot = mtop = 0;

  XtSetArg(args[cnt], XmNdefaultButtonShadowThickness, &dbst); cnt++;
  XtSetArg(args[cnt], XmNshowAsDefault, &sad); cnt++;
  XtSetArg(args[cnt], XmNhighlightThickness, &hlt); cnt++;

  XtSetArg(args[cnt], XmNshadowThickness, &st); cnt++;
  XtSetArg(args[cnt], XmNborderWidth, &bw); cnt++;
  XtSetArg(args[cnt], XmNmarginHeight, &mh); cnt++;
  XtSetArg(args[cnt], XmNmarginBottom, &mbot); cnt++;
  XtSetArg(args[cnt], XmNmarginTop, &mtop); cnt++;
  XtSetArg(args[cnt], XmNfontList, &fl); cnt++;
  XtSetArg(args[cnt], XmNlabelString, &xstr); cnt++;

  if (mb->buttons[0].button) 
    XtGetValues(mb->buttons[0].button, args, cnt);
  
  height += 2 * (st + bw + mh + dbst + sad + hlt) + 
    			       	mbot + mtop + XmStringHeight(fl, xstr);

  return(height);
}

/****************************************************************************
 *
 *			MDS functions 
 *
 ***************************************************************************/

/* loadMenuDir ****************************************************************
 * search for files ending with '.menu' under a given directory and insert them
 * into the MDS-array
 *****************************************************************************/
 
Boolean loadMenuDir(parent, menupath)
     Widget parent;
     char *menupath;
{
  char *rindex();
  register struct direct *entry;
  DIR *dirp;
  Boolean err = True;
  char fullName[256];
  register char *dot;
  
  if ((dirp = opendir(menupath)) == (DIR *) NULL) {
    setError(ERR_OPEN);
    return(False);
  }
  
  for (entry = readdir(dirp); entry != (struct direct *)NULL;
                                            entry = readdir(dirp)) {
    if ((dot = rindex(entry->d_name, '.')) == (char *) 0)
      continue;

    if (strcmp(++dot, "menu") == 0) {		/* found a menu file */
      if (makeFullName(menupath, entry->d_name, fullName, 256) < 0) {
	err = False;
	continue;
      }
      
      /* load user menus from user file */
      err = restoreFromFile(parent, fullName, 0);
    }
  }

  closedir(dirp);  
  
  return(err);
}


/* getMenuPath *************************************************************
 * provide the absolute path in which to store an application node; if the
 * file name is not an absolute path name, the path is built from the value
 * of MENUPATH or HOME in the users environment. The storage internally 
 * allocated to the result should be freed after usage.
 ****************************************************************************/

char *getMenuPath(aptr)
     ApplicationNode *aptr;
{
  char *getenv();
  char *menupath;
  char fullName[256]; 

  if (! aptr || !aptr->file) {
    setError(ERR_NULL);
    return((char *) 0);
  }

  if (aptr->a_access & MDS_TEMP) {
    aptr->_changed = False;
    return((char *) 0);
  }
  
  if (*aptr->file == '/') {			/* absolute file name */
    strcpy(fullName, aptr->file);
  } else {					/* create file name */
    if (! (menupath = getenv("MENUPATH"))) {
      if (! (menupath = getenv("HOME"))) {
	setError(ERR_NOEXIST);
	return((char *) 0);
      }    
    }
    
    if (makeFullName(menupath, aptr->file, fullName, 256) < 0) {
      setError(ERR_MAX);
      return((char *) 0);
    }
  }

  return(strCopy(fullName));
} 
	    

/* insertFunction: ***********************************************************
 * insert application function in first free position in 'ApplFunc'-array
 *****************************************************************************/

insertFunction(aptr, func, funcName)
     ApplicationNode *aptr;
     int (* func)();
     char *funcName;
{
  int i;
  
  CHECK(aptr);
  CHECK(func);
  CHECK(funcName);
  
  for (i = 1; i < MDS_MAXFUNC; i++) {
    if (aptr->ApplFunc[i].func == (int(*)()) 0) {
      aptr->ApplFunc[i].func = func;
      aptr->ApplFunc[i].funcName = strCopy(funcName);
      return(i);
    }
  }
  
	setError(ERR_MAX);
  return(-1);
}

/* setMenuActivityMask: *******************************************************
 * initialize activity mask of menu with activity mask of default item
 *****************************************************************************/

setMenuActivityMask(mptr,iptr)
     MHmenu *mptr;
     MHitem *iptr;
{
  if (! (mptr && iptr)) {
    setError(ERR_NULL);
    return;
  }
  
  AM_CPY(&mptr->activityMask, &iptr->activityMask);
}

/* isItemActive: *************************************************************
 * return TRUE if the specified item is enabled
 *****************************************************************************/

Boolean isItemActive(mptr, icnt)
     MHmenu *mptr;
     int icnt;
{
  if (! mptr) {
    setError(ERR_NULL);
    return(False);
}
  return(AM_ISSET(&mptr->activityMask, icnt)? True : False);
}

/* makeOneMenu: **************************************************************
 * create a menu with 'reasonable' defaults
 *****************************************************************************/

MHmenu *makeOneMenu(name, selCnt, selType, x, y, posType)
     char *name;
     int selCnt;
     MDS_CNT_TYPE selType;
     int x,y;
     MDS_POS_TYPE posType;
{
  register MHmenu *mptr = newMenu(name);
  setSelCnt(mptr,selCnt);
  setSelType(mptr,selType);
  setMenuPos(mptr,x,y,posType);
  
  return(mptr);
}

/* makeOneItem: **************************************************************
 * create an item with 'reasonable' defaults 
 *****************************************************************************/

MHitem *makeOneItem(mptr, name, key, retPattern, requests)
     MHmenu *mptr;
     char *name;
     char key;
     char *retPattern;
     MHiReq **requests;
{
  register MHitem *iptr = newItem(mptr,name,key);
  register i;
  
  setRetPattern(iptr, retPattern);
  
  if (requests != (MHiReq **) NULL) 
    for (i = 0; i < MDS_MAXREQ && requests[i]; i++)
      insertRequest(iptr, requests[i], i);
  
  return(iptr);
}

/* makeOneRequest: ************************************************************
 * create a request with 'reasonable' defaults
 *****************************************************************************/

MHiReq *makeOneRequest(fnct, data, initialValue, size)
     int (* fnct)();
     char *data;
     char *initialValue;
     int size;
{
  register MHiReq *req = newRequest();
  
  setRequestData(req, data);
  setRequestInitialValue(req, initialValue);
  setReqFnct(req, fnct);
  setReqCnt(req, 1, MDS_CNT_EQ);
  setReqSize(req, size);
  
  return(req);
}

/* copyMenuDescr ************************************************************
 * copy the description of a menu 
 ****************************************************************************/

copyMenuDescr(from,to)
     MHmenu *from, *to;
{
  int i;
  
  if (from == (MHmenu *)0 || to == (MHmenu *) 0)
    return(ERR_NULL);
  
  to->m_repr->reprType = from->m_repr->reprType;
  free(to->m_repr->name);
  to->m_repr->name = strCopy(from->m_repr->name);
  
  to->m_type = from->m_type;
  to->m_flags = from->m_flags;
  to->itemCnt = from->itemCnt;
  to->maxItem = from->maxItem;
  to->i_selCnt = from->i_selCnt;
  to->i_selType = from->i_selType;
  
  AM_CPY(&from->activityMask, &to->activityMask);
  
  to->m_pos.x = from->m_pos.x;
  to->m_pos.y = from->m_pos.y;
  to->m_pos.posType = from->m_pos.posType;
  
  to->m_key = from->m_key;
  
  /* copy items, not just the addresses */
  for (i = 0; i < MDS_MAXITEM; i++) {
    if (from->i_table[i] != (MHitem *)0) {
      
      if (to->i_table[i] == (MHitem *)0)
	to->i_table[i] = createItem();
      
      copyItemDescr(from->i_table[i],to->i_table[i]);
    }
    else if (to->i_table[i] != (MHitem *)0) {
      clearItem(to->i_table[i]);
      to->i_table[i] = (MHitem *) 0;
    }
  }
  
  free(to->m_hlp->hlpmsg);
  to->m_hlp->hlpmsg = strCopy(from->m_hlp->hlpmsg);
  to->m_hlp->hlpFnct = from->m_hlp->hlpFnct;
  
  return(ERR_NOERR);
}

/* insertActMasks *************************************************************
 * insert activity masks for menu and items
 *****************************************************************************/

insertActMasks(mptr,am)
     MHmenu *mptr;
     act_mask *am;
{
  register i,cnt;
  ApplicationNode *aptr = currentApplication();
  
  if (Access(aptr,MDS_CHNGACC) == False)
    return(setError(ERR_ACCESS));
  
  if (mptr == (MHmenu *) 0)
    return(setError(ERR_NULL));
  
  AM_CPY(&am[0], &mptr->activityMask);
  
  for (cnt = 1,i = 0; i < MDS_MAXITEM && cnt <= mptr->itemCnt; i++) {
    if (mptr->i_table[i] == (MHitem *) 0)
      continue;
    
    AM_CPY(&am[cnt], &mptr->i_table[i]->activityMask);
    cnt++;
  }
  BYE;
}

/* MenusToSave: **************************************************************
 * look if one of the menu-descriptions of the current application has 
 * been changed
 *****************************************************************************/

Boolean MenusToSave(aptr)
     ApplicationNode *aptr;
{
  if (! aptr) {
    setError(ERR_NULL);
    return(False);
  }
  return(aptr->_changed);
}

/* buildMenuFromStrings: ******************************************************
 * build a menu whose items are made of strings provided in the itemList 
 * argument; the return value of an item is defined by the returnList
 *****************************************************************************/

MHmenu *buildMenuFromStrings(name, type, itemList, returnList, cnt)
     char *name;
     MDS_MENU_TYPE type;
     char *itemList[];		/* holds the list of item names */
     char *returnList[];        /* holds the corresponding return values; 
				 * if NULL the item index is returned */
     int cnt;			/* the number of tokens in the list */
{
  MHitem *iptr;
  MHmenu *mptr = createMenu();
  char str[100];
  register i;
  
  if (! mptr)
    return((MHmenu *) 0);
  
  initMenu(mptr, type, name);
  mptr->m_pos.x = mptr->m_pos.y = 10;
  mptr->m_hlp->hlpmsg = strCopy("A system generated menu: please select.");
  mptr->m_key = EOS;
  
  for (i = 0; i < MIN(cnt, MDS_MAXITEM); i++) {
    iptr = newItem(mptr,itemList[i], EOS);
    if (! returnList)
      iptr->i_return->retPattern = strCopy(sprintf(str, "%d", i));
    else
      iptr->i_return->retPattern = strCopy(sprintf(str, "%s ", returnList[i]));
  }
  
  return((mptr->itemCnt > 0) ? mptr : (MHmenu *) 0);
}

/* buildMenuList *************************************************************
 * steps through an application node and collects all menus; their names are 
 * put into the array pointed to by list; the number of collected menus is
 * returned 
 ****************************************************************************/

buildMenuList(aptr, mlist)
     ApplicationNode *aptr;
     MHmenu *mlist[MDS_MAXMENU];
{
  int mcnt = 0, i = 0;
  
  if (!aptr || !mlist) {
    setError(ERR_NULL);
    return(-1);
  }

  while (i < MDS_MAXMENU && mcnt < aptr->menuCnt) {
    if (aptr->m_table[i])
      mlist[mcnt++] = aptr->m_table[i];
    i++;
  }

  return(mcnt);
}

/* isPopup ******************************************************************
 * returns 'true' if menu is a popup menu or popup list
 ****************************************************************************/

Boolean isPopup(mptr)
     MHmenu *mptr;
{
  return((mptr->m_type == MDS_M_POPUP || mptr->m_type == MDS_L_POPUP) ? True : False);
}

/* isPulldown ***************************************************************
 * returns 'true' if menu is a pulldown menu or pulldown list
 ****************************************************************************/

Boolean isPulldown(mptr)
     MHmenu *mptr;
{
  return((mptr->m_type == MDS_M_PULLD || mptr->m_type == MDS_L_PULLD)? True : False);
}

/* initMDS: *******************************************************************
 * initialize the global MDS-array if not initialized
 *****************************************************************************/
     
initMDS()
{
  static initialised = 0;
  
  if (initialised)
    return;
  
  initialised = 1;
  
  initMDSarray();
}

/* setWorkApplication: ******************************************************
 * create an internal temporary working application node and set 'curAppl' 
 ****************************************************************************/

setWorkApplication()
{
  if (! exWorkApplication) {
    exWorkApplication = createApplNode();
    initApplNode(exWorkApplication, "__%%work-application%%__");
    setAccess(exWorkApplication, MDS_CHNGACC | MDS_TEMP);  
    insertApplNode(exWorkApplication);
  }
  setCurrentApplication(exWorkApplication);
}

/* deleteMenu: **************************************************************
 * delete the specified menu structure from a given application node (allowed 
 * to owner of application node or if change access is set for application 
 * node)
 ****************************************************************************/

Boolean deleteMenu(aptr, m)
     ApplicationNode *aptr;
     MHmenu *m;
{
  register i;
  
  if (m == (MHmenu *)0) {
    setError(ERR_NULL);
    return(False);
  }
  
  if (Access(aptr,MDS_CHNGACC) == False) {
    setError(ERR_ACCESS);
    return(False);
  }
  
  for (i = 0; i < MDS_MAXMENU; i++) {
    if (aptr->m_table[i] != m)
      continue;
    
    clearMenu(m);
    
    aptr->menuCnt--;
    aptr->m_table[i] = (MHmenu *) 0;
    aptr->_changed = True;
    setError(ERR_NOERR);
    return(True);
  }
  setError(ERR_NULL);
  return(False);
}

/* deleteWorkMenu: ***********************************************************
 * delete menu from working application node
 ****************************************************************************/

Boolean deleteWorkMenu(m)
     MHmenu *m;
{
  return(deleteMenu(exWorkApplication, m));
}

/* deleteItem: ***************************************************************
 * delete the specified item structure (allowed to owner of current 
 * application node or if change access is set for current application node)
 *****************************************************************************/

Boolean deleteItem(m,item)
     MHmenu *m;                   /* menu under current application node */
     MHitem *item;
{
  register i,j,k;
  ApplicationNode *aptr;

  CHECK(item);
  
  if (! (aptr = currentApplication())) {
    setError(ERR_NULL);
    return(False);
}
  if (Access(aptr, MDS_CHNGACC) == False) {
    setError(ERR_ACCESS);
    return(False);
  }

  for (i = 0; i < MDS_MAXITEM; i++) 
    if (m->i_table[i] == item) {
      clearItem(item);
      m->i_table[i] = (MHitem *) 0;
      m->itemCnt--;
      for (j = i; j < MDS_MAXITEM - 1; j++) {
	m->i_table[j] = m->i_table[j+1];
	m->i_table[j + 1] = (MHitem *) 0;
	if (AM_ISSET(&m->activityMask, j + 1))
	  AM_SET(&m->activityMask, j);
	else
	  AM_CLR(&m->activityMask, j);
	  
	if (m->i_table[j]) {
	  for (k = i; k < MDS_MAXITEM; k++)
	    if (AM_ISSET(&m->i_table[j]->activityMask, k + 1))
	      AM_SET(&m->i_table[j]->activityMask, k);
	    else
	      AM_CLR(&m->i_table[j]->activityMask, k);
	}
      }
      aptr->_changed = True;
      setError(ERR_NOERR);
      return(True);
    }
  setError(ERR_NOEXIST);
  return(False);
}

/* getMenuDescrByName: *******************************************************
 * search for menu with given name in current application node; return 
 * menu pointer if found
 *****************************************************************************/

MHmenu *getMenuDescrByName(name)
     char *name;
{
  register i;
  register MHmenu *mptr;
  register ApplicationNode *aptr;

  if (name == (char *) 0) {
    setError(ERR_NULL);
    return((MHmenu *) 0);
  }

  if (! (aptr = currentApplication())) {
    setError(ERR_NULL);
    return((MHmenu *) 0);
  }
  
  for (i = 0; i < MDS_MAXMENU; i++) 
    if (mptr = aptr->m_table[i])
      if (*(mptr->m_repr->name) && 
	  strcmp(mptr->m_repr->name,name) == 0) {
	setError(ERR_NOERR);
	return(aptr->m_table[i]);
  }
  setError(ERR_NOEXIST);
  return((MHmenu *) 0);
}

/* newRequest: ***************************************************************
 * create and initialize request structure with 'reasonable' values
 ****************************************************************************/

MHiReq *newRequest()
{
  MHiReq *reqPtr = (MHiReq *) Malloc(sizeof(MHiReq));
  
  reqPtr->reqFnct = (int (*)()) reqViaWnd;
  reqPtr->flags = MH_MDSFUNCTION;
  reqPtr->reqData = (char *) 0;
  reqPtr->initialValue = (char *) 0;
  reqPtr->prelimInitialValue = (char *) 0;
  reqPtr->cnt = 0;
  reqPtr->size = MDS_NAMELEN;
  reqPtr->reqCntType = MDS_CNT_NULL;
  reqPtr->reqPtr = (caddr_t) 0;
  reqPtr->returnValue = (char *) 0;
  return(reqPtr);
}

/* insertRequest: ************************************************************
 * insert request into given request-array slot of item
 ****************************************************************************/

insertRequest(iptr, req, rcnt)
     MHitem *iptr;
     MHiReq *req;
     int rcnt;
{
  register ApplicationNode *aptr;

  if (! (aptr = currentApplication()))
    return(setError(ERR_NULL));
  
  if (Access(aptr, MDS_CHNGACC) == False)
    return(setError(ERR_ACCESS));
  
  if (rcnt < 0 || rcnt > MDS_MAXREQ)
    return(setError(ERR_VALUE));
  
  if (iptr == (MHitem *) 0 || req == (MHiReq *)0)
    return(setError(ERR_NULL));
  
  if (iptr->i_requests[rcnt]) 
    deleteRequest(iptr, rcnt);
  iptr->i_requests[rcnt] = req;
  iptr->i_return->reqCnt++;
  
  return(setError(ERR_NOERR));
}

/* setRequestData: ***********************************************************
 * set data for the request function: e.g. in the case of reqViaWindow a
 * message, in the case of a menu or form the menu's or form's name 
 *****************************************************************************/

setRequestData(req, data)
     MHiReq *req;
     char *data;
{
  CHECK(req);
  
  free(req->reqData);
  req->reqData = strCopy(data);
  
  BYE;
}

/* setRequestInitialValue: ***************************************************
 * assign initial value to request (e.g. default string displayed in a 
 * 'reqViaWin'-window)
 ****************************************************************************/

setRequestInitialValue(req, initialValue)
     MHiReq *req;
     char *initialValue;
{
  CHECK(req);
  
  if (req->initialValue)
    free(req->initialValue);
  req->initialValue = strCopy(initialValue);
  
  BYE;
}

/* setReqFnct: **************************************************************
 * set request function and specify flag (user defined function/system 
 * function)
 *****************************************************************************/

setReqFnct(req, fnct)
     MHiReq *req;
     int (* fnct)();
{
  CHECK(req);
  
  req->reqFnct = fnct;
  
  if (mds_function(fnct))
    req->flags = MH_MDSFUNCTION;
  else
    req->flags = MH_USERFUNCTION;
  
  BYE;
}

/* setReqCnt: ****************************************************************
 * set token count, indicating number of tokens to be requested by a request
 ****************************************************************************/

setReqCnt(req, cnt, type)
     MHiReq *req;
     int cnt;
     MDS_CNT_TYPE type;
{
  CHECK(req);
  
  if (legalType ("MDS_CNT_TYPE",(int) type) == False)
    return(setError(ERR_TYPE));
  
  req->cnt = cnt;
  req->reqCntType = type;
  
  BYE;
}

/* setReqSize: ***************************************************************
 * set request size (in case of reqViaWindow the length of the entry)
 *****************************************************************************/

setReqSize(req, size)
     MHiReq *req;
     int size;
{
  CHECK(req);
  
  if (size <= 0 || size > MDS_NAMELEN)
    return(setError(ERR_SIZE));
  
  req->size = size;
  
  BYE;
}

/* setToggleItem: ************************************************************
 * define item as toggle item, used to switch between two states
 *****************************************************************************/

setToggleItem(iptr, onoff)
     MHitem *iptr;
     Boolean onoff;
{
  CHECK(iptr);

  if (onoff)
    iptr->flag |= MH_TOGGLEITEM;
  else 
    iptr->flag &= ~MH_TOGGLEITEM;
  return(setError(ERR_NOERR));
}

/* setToggle: *****************************************************************
 * a toggle item can be displayed with a tick to feed back the execution of a 
 * toggle item 
 *****************************************************************************/

setToggle(iptr, onoff)
     MHitem *iptr;
     Boolean onoff;
{
  CHECK(iptr);

  if (onoff)
    iptr->flag |= MH_TICKON;
  else 
    iptr->flag &= ~MH_TICKON;
  return(setError(ERR_NOERR));
}

/*****************************************************************************
 *			generate components
 ****************************************************************************/

/* newRetPattern: ************************************************************
 * create return structure, insert return pattern and calculate the request 
 * count from the pattern; return pointer to return structure
 ****************************************************************************/

MHiRet *newRetPattern(pattern)
     char *pattern;
{
  register MHiRet *ptr = (MHiRet *) Malloc(sizeof(MHiRet));
  ptr->retPattern = strCopy(pattern);
  ptr->reqCnt  = countRequestsInPattern(pattern);
  return(ptr);
}

/* new_iRepr: ***************************************************************
 * create representation structure and assign given values to it; return 
 * pointer
 ****************************************************************************/

MHiRepr *new_iRepr(name, key)
     char *name;		/* the actual representation */
     char key;
{
  register MHiRepr *ptr = (MHiRepr *) Malloc(sizeof(MHiRepr));
  
  ptr->name = strCopy(name);
  ptr->key = key;
  return(ptr);
}

/*****************************************************************************
 *			set components 
 ****************************************************************************/

/* setAccess ******************************************************************
 * specify access type (no access, read/change access, save/temporary access, 
 * system menu flag set) for appcliation node
 *****************************************************************************/

setAccess(aptr, access)
     ApplicationNode *aptr;
     int access;
{
  CHECK(aptr);
  
  aptr->a_access = access;
  return(setError(ERR_NOERR));
}

/* setMenuInitialActivityMask ************************************************
 * mark each item with a '0' in the corresponding array position as 
 * non-selectable
 ****************************************************************************/

setMenuInitialActivityMask(mptr, actVal)
     MHmenu *mptr;
     int actVal[];          /* array containing a '0'(non-selectable) or 
			       '1'(selectable) for each item of menu */
{
  register i;

  CHECK(mptr);

  for (i = 0; i < mptr->itemCnt; i++)
    if (actVal[i])
    AM_SET(&mptr->activityMask, i);
  else 
    AM_CLR(&mptr->activityMask, i);
  return(setError(ERR_NOERR));
}

/* setItemActivityMask ******************************************************
 * mark each item with a '0' in the corresponding array position as
 * non-selectable if input item is selected
 ****************************************************************************/

setItemActivityMask(mptr, iptr, actVal)
     MHmenu *mptr;
     MHitem *iptr;
     int actVal[];
{  
  register i;

  CHECK(mptr);
  CHECK(iptr);

  for (i = 0; i < mptr->itemCnt; i++)
    if (actVal[i])
    AM_SET(&iptr->activityMask, i);
  else 
    AM_CLR(&iptr->activityMask, i);
  return(setError(ERR_NOERR));
}

/* setDefaultItem ************************************************************
 * item is selected (inverse representation) when menu appears on the screen
 ****************************************************************************/

setDefaultItem(iptr)
     MHitem *iptr;
{
  CHECK(iptr);
  
  iptr->flag |= MH_DEFAULTITEM;
  
  BYE;
}

/* resetDefaultItem **********************************************************
 * item is not selected (inverse representation) when menu appears on the 
 * screen
 ****************************************************************************/

resetDefaultItem(iptr)
     MHitem *iptr;
{
  CHECK(iptr);
  
  iptr->flag &= ~MH_DEFAULTITEM;
  
  BYE;
}

/* setSelCnt ****************************************************************
 * assign selection count to menu which specifies how many items can be 
 * selected 
 ****************************************************************************/

setSelCnt(mptr,cnt)
     MHmenu *mptr;
     int cnt;
{
  CHECK(mptr) ;
  
  mptr->i_selCnt = cnt;
  
  BYE;
}

/* setMaxItem ****************************************************************
 * assign max item count to menu which specifies how many items can be 
 * displayed 
 ****************************************************************************/

setMaxItem(mptr, cnt)
     MHmenu *mptr;
     int cnt;
{
  CHECK(mptr) ;
  
  mptr->maxItem = cnt;
  
  BYE;
}

/* setSelType ****************************************************************
 * assign item selection type to menu, i.e. if number of selectable items is 
 * ==, <, <=, > or >= than the selection count
 ****************************************************************************/

setSelType(mptr, type)
     MHmenu *mptr;
     MDS_CNT_TYPE type;
{
  CHECK(mptr);
  
  if (legalType ("MDS_CNT_TYPE",(int) type) == False)
    return(setError(ERR_TYPE));
  
  mptr->i_selType = type;
  
  BYE;
}

/* setMenuRepresentation *****************************************************
 * assign name, type of representation (text, icon) to menu 
 *****************************************************************************/

setMenuRepresentation(mptr,name,rtype)
     MHmenu *mptr;
     char *name;
     MDS_REPR_TYPE rtype;
{
  CHECK(mptr);
  CHECK(mptr->m_repr);
  
  if (legalType("MDS_REPR_TYPE",(int) rtype) == False)
    return(setError(ERR_TYPE));
  
  if (name) {
    free(mptr->m_repr->name);
    mptr->m_repr->name = strCopy(name);
  }
  
  if (rtype != MDS_RTYP_NULL)
    mptr->m_repr->reprType = rtype;
  
  BYE;
}

/* setMenuType ***************************************************************
 * assign menu type (popup menu/popup list/pulldown menu/pulldown list) 
 ****************************************************************************/

setMenuType(mptr, type)
     MHmenu *mptr;
     MDS_MENU_TYPE type;
{
  CHECK(mptr);
  
  if (legalType("MDS_MENU_TYPE", (int)type) == False)
    return(setError(ERR_TYPE));
  
  mptr->m_type = type;
  
  BYE;
}

/* setMenuCoordinates *********************************************************
 * assign position coordinates to menu 
 *****************************************************************************/

setMenuCoordinates(mptr, x, y)
     MHmenu *mptr;
     int x, y;
{
  CHECK(mptr);
  
  mptr->m_pos.x = x; mptr->m_pos.y = y;
  
  BYE;
}

/* setMenuPosType ************************************************************
 * assign position type (absolute/cursor relative) to menu 
 ****************************************************************************/

setMenuPosType(mptr, posType)
     MHmenu *mptr;
     MDS_POS_TYPE posType;
{
  CHECK(mptr);
  
  if (legalType("MDS_POS_TYPE",(int) posType) == False)
    return(setError(ERR_TYPE));
  
  mptr->m_pos.posType = posType;
  
  BYE;
}

/* setMenuPos ****************************************************************
 * assign position coordinates and position type (absolute/cursor relative) 
 * to menu 
 ****************************************************************************/

setMenuPos(mptr, x, y, posType)
     MHmenu *mptr;
     int x, y;
     MDS_POS_TYPE posType;
{
  CHECK(mptr);
  
  if (legalType("MDS_POS_TYPE",(int) posType) == False)
    return(setError(ERR_TYPE));
  
  mptr->m_pos.x = x; mptr->m_pos.y = y;
  mptr->m_pos.posType = posType;
  
  BYE;
}

/* setFile: ******************************************************************
 * set file name in which application node is to be saved
 *****************************************************************************/

setFile(aptr,string)
     ApplicationNode *aptr;
     char *string;
{
  CHECK(aptr);
  
  free(aptr->file);
  aptr->file = strCopy(string);
  
  BYE;
}

/* setRetPattern: ************************************************************
 * insert return pattern including a '$' for each request; the request count is
 * kept according to actually specified requests (see 'insertRequest')
 ****************************************************************************/

setRetPattern(iptr,ret)
     MHitem *iptr;
     char *ret;
{
  CHECK(iptr && ret);
  
  if (iptr->i_return->retPattern)
    free(iptr->i_return->retPattern);
  iptr->i_return->retPattern = strCopy(ret);

  BYE;
}

/* Access: ********************************************************************
 * check access rights for application node
 *****************************************************************************/

Boolean Access(aptr, access)
     ApplicationNode *aptr;
     int access;
{
  if (! aptr) { 
    setError(ERR_NULL);
    return(False);
  }
  
  if (aptr->a_access & access)
    return(True);
  
  return(False);
}
