/************************************************************************
 *                                                                      *
 *    Copyright (c) 1991 Wilhelm Valder, Regine Freitag, Wolfgang Dzida *
 *                        All rights reserved                           *
 *                                                                      *
 *                                                                      *
 *    Permission to copy this program or any part of this program is    *
 *    granted in case of personal, non-commercial use only. Any use     *
 *    for profit or other commercial gain without written permissions   *
 *    of the authors is prohibited.                                     *
 *                                                                      *
 *    You may freely distribute this program to friends, colleagues,    *
 *    etc.  As a courtesy we would appreciate it if you left our names  *
 *    on the code as the authors.                                       *
 *    We also, of course, disavow any liabilities                       *
 *    for any bizarre things you may choose to do with the system.      *
 *                                                                      *
 *    We would appreciate though your letting us know about any         *
 *    extensions, modifications, project suggestions, etc.              *
 *    Please mail all bugs, comments, complaints and suggestions to:    *
 *                                                                      *
 *                                                                      *
 *    GMD (German National Research Center for Computer Science)        *
 *    Institut fuer Systemtechnik (F2G2)                                *
 *    P.O. Box 1240                                                     *
 *    Schloss Birlinghoven                                              *
 *    D-5205 St. Augustin 1, West Germany                               *
 *                                                                      *
 *    email:    dzida@kmx.gmd.dbp.de                                    *
 *              freitag@gmdzi.uunet                                     *
 *                                                                      *
 ************************************************************************/
/***************************************************************************
 * This file contains the program to generate the system menus of the 
 * menu editor; the generated application node "edit menus" is stored in 
 * file "editMenus.menu".
 *************************************************************************/

#include "editNames.h"
#include <exBasics.h>
#include <exFonts.h>
#include <exErrors.h> 
#include <exMenu.h>

static char helpText[200];

static ApplChange(/* parms */);
static editMenuType(/* parms */);
static editMenuPos1(/* parms */);
static editMenuPos2(/* parms */);
static editMenuActMask(/* parms */);
static editSelType(/* parms */);
static editSelCnt(/* parms */);
static editRequest(/* parms */);
static adminSave(/* parms */);
static adminQuit(/* parms */);
static adminNewMenu(/* parms */);
static adminNewSet(/* parms */);
static adminDuplSet(/* parms */);
static adminDuplMenu(/* parms */);
static adminPreview(/* parms */);
static adminUserFunct(/* parms */);
static editAccessType(/* parms */);
static editMenuHelp(/* parms */);
static newApplList(/* parms */);
static newMenuList(/* parms */);

main(argc, argv)
     int argc;
     char *argv[]; 
{
  Widget top;
  Boolean createEditMenus();

#ifdef DEBUG
  malloc_debug(2);
#endif

  top = XtInitialize(NULL, "menu editor menus", (XrmOptionDescRec *) NULL, 
		     0, &argc, argv);

  exInitLayout(XtDisplay(top));
  initMDS();

  if (!createEditMenus(top))
    printf("application node 'edit menus' not generated!\n");

  exit(0);
}

/* createEditMenus *********************************************************
 * this file contains the information to generate the menus needed
 * to edit a menudescription. The generated application node "edit menus" 
 * is stored in file "editMenus.menu"
 ***************************************************************************/

Boolean createEditMenus(parent)
     Widget parent;
{
  register i;

  ApplicationNode *EDIT_MENUS = (ApplicationNode *) 0;
  MHmenu *editMenu;
  MHitem *editItem;
  MHiReq *Req;
  int actmask3[3], actmask6[6], actmask11[11];
    
  /* new Application Node for edit menus */
  EDIT_MENUS = createApplNode();
  initApplNode(EDIT_MENUS, EDITMENUS);
  setAccess(EDIT_MENUS, MDS_CHNGACC | MDS_SYS);
  
  if (insertApplNode(EDIT_MENUS) == ERR_MAX) {
    printf("cannot create Application Node; too many nodes\n");
    return(False);
  }

  insertFunction(EDIT_MENUS, adminSave, "adminSave");
  insertFunction(EDIT_MENUS, adminQuit, "adminQuit");
  insertFunction(EDIT_MENUS, adminNewSet, "adminNewSet");
  insertFunction(EDIT_MENUS, adminDuplSet, "adminDuplSet");
  insertFunction(EDIT_MENUS, adminDuplMenu, "adminDuplMenu");
  insertFunction(EDIT_MENUS, newApplList, "newApplList");
  insertFunction(EDIT_MENUS, adminUserFunct, "adminUserFunct");
  insertFunction(EDIT_MENUS, ApplChange, "ApplChange");
  insertFunction(EDIT_MENUS, adminNewMenu, "adminNewMenu");
  insertFunction(EDIT_MENUS, adminPreview, "adminPreview");
  insertFunction(EDIT_MENUS, newMenuList, "newMenuList");
  insertFunction(EDIT_MENUS, editMenuType, "editMenuType");
  insertFunction(EDIT_MENUS, editAccessType, "editAccessType");
  insertFunction(EDIT_MENUS, editMenuPos1, "editMenuPos1");
  insertFunction(EDIT_MENUS, editMenuPos2, "editMenuPos2");
  insertFunction(EDIT_MENUS, editMenuActMask, "editMenuActMask");
  insertFunction(EDIT_MENUS, editSelCnt, "editSelCnt");
  insertFunction(EDIT_MENUS, editSelType, "editSelType");
  insertFunction(EDIT_MENUS, editRequest, "editRequest");
  insertFunction(EDIT_MENUS, editMenuHelp, "editMenuHelp");

  /*******************************************************************
   *                   create edit_menu QUIT                      *
   ******************************************************************/

  /* create edit menu QUIT for Application Node EDIT_MENUS */
  editMenu = newMenu("quit");

  setSelCnt(editMenu, 1);                          /* set selection count */
  setSelType(editMenu, MDS_CNT_EQ);                /* set selection type */
  setMenuType(editMenu, MDS_M_PULLD);              /* set menu type */
  setMenuFlag(editMenu, MDS_READACC);              /* set access rights */
  sprintf(helpText, "Option to quit the menu editor.");
  editMenu->m_hlp->hlpmsg = strCopy(helpText);
  
  /* create items for edit menu QUIT */

  editItem = newItem(editMenu, "quit", EOS);
  setRetPattern(editItem, "$");                    /* set return pattern */
  Req = makeOneRequest(adminQuit, (char *) 0, (char *) 0, 0);
  insertRequest (editItem, Req, 0);                /* set request */
  sprintf(helpText, "Quit the menu editor.");
  editItem->i_hlp->hlpmsg = strCopy(helpText);
  
  /*******************************************************************
   *                   create edit_menu MENU SETS                    *
   ******************************************************************/

  /* create edit menu MENU SETS for Application Node EDIT_MENUS */
  editMenu = newMenu(MENUSETS);

  setSelCnt(editMenu,1);                            /* set selection count */
  setSelType(editMenu,MDS_CNT_EQ);                  /* set selection type */
  setMenuType(editMenu, MDS_M_PULLD);               /* set menu type */ 
  setMenuFlag(editMenu, MDS_READACC);               /* set access rights */
  sprintf(helpText, "Options to manage menu sets");
  editMenu->m_hlp->hlpmsg = strCopy(helpText);

  /* create items for edit menu MENU SETS */

  editItem = newItem(editMenu, "load set", EOS);
  setRetPattern(editItem, "$ $");                      /* set return pattern */
  Req = makeOneRequest(reqViaMenu, APPLLISTMENU, (char *) 0, 0);
  insertRequest (editItem, Req, 0);                    /* set request */
  Req = makeOneRequest(newApplList, (char *) 0, (char *) 0, 0);
  insertRequest (editItem, Req, 1);
  sprintf(helpText, "Load a menu set and show the first menu.\nYou get a menu containing\nthe list of available menu sets\nfrom which you can select the desired one.");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  editItem = newItem(editMenu, "new set", EOS);
  setRetPattern(editItem, "$ $");
  Req = makeOneRequest(reqViaWnd, "menu set name:", (char *) 0, E_ENTRYNAME);
  insertRequest (editItem, Req, 0);
  Req = makeOneRequest(adminNewSet, "1", (char *) 0, 0);
  insertRequest (editItem, Req, 1);
  sprintf(helpText, "Define a new menu set.\nYou are requested to enter\nthe new name for the menu set.");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  editItem = newItem(editMenu, "save set", EOS);
  setRetPattern(editItem, "$");                    /* set return pattern */
  Req = makeOneRequest(adminSave, (char *) 0, (char *) 0, 0);
  insertRequest (editItem, Req, 0);                /* set request */
  sprintf(helpText, "Save the current menu set to a file.");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  editItem = newItem(editMenu, "save set as", EOS);
  setRetPattern(editItem, "$ $");                    /* set return pattern */
  Req = makeOneRequest(reqViaWnd, "menu set name:", (char *) 0, E_ENTRYNAME);
  insertRequest (editItem, Req, 0);                /* set request */
  Req = makeOneRequest(adminDuplSet, "3", (char *) 0, 0);
  insertRequest (editItem, Req, 1);
  sprintf(helpText, "Duplicate the current menu set\nand store it under\na different name.\nYou are requested to enter\nthe new name for the menu set.");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  editItem = newItem(editMenu, "delete sets", EOS);
  setRetPattern(editItem, "$ $");
  Req = makeOneRequest(reqViaMenu, APPLDELMENU, (char *) 0, 0);
  setReqCnt(Req, 1, MDS_CNT_GE);
  insertRequest (editItem, Req, 0);
  Req = makeOneRequest(newApplList, (char *) 0, (char *) 0, 0);
  insertRequest (editItem, Req, 1);
  sprintf(helpText, "Delete a menu set and the corresponding file.\nYou get a menu containing\n the list of your menu sets\nfrom which you can select the desired one.");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  editItem = newItem(editMenu, "------------------", EOS);

  editItem = newItem(editMenu, "edit attributes", EOS);
  setRetPattern(editItem, "$ $");
  Req = makeOneRequest(reqViaMenu, CHANGE, (char *) 0, 0);
  insertRequest (editItem, Req, 0);
  setReqCnt(Req, 1, MDS_CNT_GE);
  Req = makeOneRequest(ApplChange, "6", (char *) 0, 0);
  insertRequest (editItem, Req, 1);
  sprintf(helpText, "Manipulate attributes such as access rights,\nname and file of the current menu set.\nYou get a submenu containing\nan option for each attribute.");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  editItem = newItem(editMenu, "handle functions", EOS);
  setRetPattern(editItem, "$ $");
  Req = makeOneRequest(reqViaMenu, FUNCMENU, (char *) 0, 0);
  setReqCnt(Req, 1, MDS_CNT_EQ);
  insertRequest (editItem, Req, 0);
  Req = makeOneRequest(adminUserFunct, "7", (char *) 0, 0);
  insertRequest (editItem, Req, 1);
  sprintf(helpText, "Handle application functions for the current menu set.\nYou get a submenu containing\nthe options show, define and delete applying to application functions.");
  editItem->i_hlp->hlpmsg = strCopy(helpText);
  
  /*******************************************************************
   *                   create edit_menu MENUS                        *
   ******************************************************************/

  /* create edit menu MENUS for Application Node EDIT_MENUS */
  editMenu = newMenu(MENUS);

  setSelCnt(editMenu,1);                            /* set selection count */
  setSelType(editMenu,MDS_CNT_EQ);                  /* set selection type */
  setMenuType(editMenu, MDS_M_PULLD);               /* set menu type */ 
  setMenuFlag(editMenu, MDS_READACC);               /* set access rights */
  sprintf(helpText, "Options to manage menus.");
  editMenu->m_hlp->hlpmsg = strCopy(helpText);

  /* create items for edit menu MENUS */

  editItem = newItem(editMenu, "load menu", EOS);
  setRetPattern(editItem, "$");
  Req = makeOneRequest(reqViaMenu, MENULISTMENU, (char *) 0, 0);
  insertRequest (editItem, Req, 0);
  sprintf(helpText, "Load a menu from the current menu set.\nYou get a menu containing\n the list of available menus\nfrom which you can select the desired one.");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  editItem = newItem(editMenu, "new menu", EOS);
  setRetPattern(editItem, "$");
  Req = makeOneRequest(adminNewMenu, (char *) 0, (char *) 0, 0);
  insertRequest (editItem, Req, 0);
  sprintf(helpText, "Define a new menu for the current menu set.\nYou are requested to enter the new name for the menu.");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  editItem = newItem(editMenu, "save menu", EOS);
  setRetPattern(editItem, "$");
  Req = makeOneRequest(adminSave, (char *) 0, (char *) 0, 0);
  insertRequest (editItem, Req, 0);
  sprintf(helpText, "Save the current menu set to a file.");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  editItem = newItem(editMenu, "save menu as", EOS);
  setRetPattern(editItem, "$ $");                    /* set return pattern */
  Req = makeOneRequest(reqViaWnd, "new menu name:", (char *) 0, E_ENTRYNAME);
  insertRequest (editItem, Req, 0);                /* set request */
  Req = makeOneRequest(adminDuplMenu, "3", (char *) 0, 0);
  insertRequest (editItem, Req, 1);                /* set request */
  sprintf(helpText, "Duplicate the current menu\nand store it in the current menu set.\nYou are requested to enter the new name for the menu.");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  editItem = newItem(editMenu, "save menu in", EOS);
  setRetPattern(editItem, "$");                    /* set return pattern */
  Req = makeOneRequest(reqViaMenu, APPLCOPYMENU, (char *) 0, 0);
  insertRequest (editItem, Req, 0);                /* set request */
  sprintf(helpText, "Store the current menu into a different menu set.\nYou get a menu containing\nthe list of available menu sets\nfrom which you can select the desired one.");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  editItem = newItem(editMenu, "delete menus", EOS); 
  setRetPattern(editItem, "$ $"); 
  Req = makeOneRequest(reqViaMenu, MENUDELMENU, (char *) 0, 0); 
  insertRequest (editItem, Req, 0);
  setReqCnt(Req, 1, MDS_CNT_GE);
  Req = makeOneRequest(newMenuList, (char *) 0, (char *) 0, 0); 
  insertRequest (editItem, Req, 1);
  sprintf(helpText, "Delete menus from the current menu set.\nYou get a menu containing\nthe list of menus\nfrom which you can select the desired one.");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  editItem = newItem(editMenu, "-------------", EOS);

  editItem = newItem(editMenu, "menu preview", EOS);
  setRetPattern(editItem, "$");
  Req = makeOneRequest(adminPreview, (char *) 0, (char *) 0, 0);
  insertRequest (editItem, Req, 0);
  sprintf(helpText, "Simulate execution of the current menu\nwithout performing real actions.");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  editItem = newItem(editMenu, "copy menu", EOS);
  setRetPattern(editItem, "$");
  Req = makeOneRequest(reqViaMenu, MENUCOPY, (char *) 0, 0);
  insertRequest (editItem, Req, 0);
  sprintf(helpText, "Copy a menu from the current menu set\nto a different menu set.\nYou get a menu containing\nthe list of available menu sets\nfrom which you can select the desired one.");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  /*setMask(actmask11, 12, 1);
  actmask11[0] = 0;
  actmask11[6] = 0;
  setMenuInitialActivityMask(editMenu, actmask11);  * set activity mask */

  /*******************************************************************
   *                   create edit_menu MENU_TYPE                    *
   ******************************************************************/

  /* create edit menu MENU_TYPE for Application Node EDIT_MENUS */
  editMenu = newMenu(MENUTYPE);

  setSelCnt(editMenu,1);                            /* set selection count */
  setSelType(editMenu,MDS_CNT_EQ);                  /* set selection type */
  setMenuType(editMenu, MDS_M_PULLD);               /* set menu type */
  setMenuFlag(editMenu, MDS_READACC);               /* set access rights */
  sprintf(helpText, "Options to change the menu type.");
  editMenu->m_hlp->hlpmsg = strCopy(helpText);
  
  /* create items for edit menu MENU_TYPE */
  editItem = newItem(editMenu, "pull-down menu", EOS);
  setRetPattern(editItem, "$");                  /* set return pattern */
  Req = makeOneRequest(editMenuType, "pulldownmenu", (char *) 0, 0);
  insertRequest (editItem, Req, 0);
  sprintf(helpText, "Make the current menu a pull-down menu.");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  editItem = newItem(editMenu, "pop-up menu", EOS);
  setRetPattern(editItem, "$");
  Req = makeOneRequest(editMenuType, "popupmenu", (char *) 0, 0);
  insertRequest (editItem, Req, 0);
  sprintf(helpText, "Make the current menu a pop-up menu.");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  editItem = newItem(editMenu, "pull-down list", EOS);
  setRetPattern(editItem, "$");
  Req = makeOneRequest(editMenuType, "pulldownlist", (char *) 0, 0);
  insertRequest (editItem, Req, 0);
  sprintf(helpText, "Make the current menu a pull-down list.");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  editItem = newItem(editMenu, "pop-up list", EOS);
  setRetPattern(editItem, "$");
  Req = makeOneRequest(editMenuType, "popuplist", (char *) 0, 0);
  insertRequest (editItem, Req, 0);
  sprintf(helpText, "Make the current menu a pop-up list.");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  /*******************************************************************
   *                   set values of menu MENU_ACCESS                *
   ******************************************************************/

  /* create edit menu MENU_ACCESS for Application Node EDIT_MENUS */
  editMenu = newMenu(MENUACCESS);
  setSelCnt(editMenu,1);                         /* set selection count */
  setSelType(editMenu,MDS_CNT_EQ);               /* set selection type */
  setMenuType(editMenu, MDS_M_PULLD);            /* set menu type */
  setMenuFlag(editMenu, MDS_READACC);            /* set access rights */
  sprintf(helpText, "Options to change access rights of a menu.");
  editMenu->m_hlp->hlpmsg = strCopy(helpText);

  /* create items for edit menu MENU_ACCESS */

  editItem = newItem(editMenu, "read access", EOS);
  setRetPattern(editItem, "$");               /* set return pattern */
  Req = makeOneRequest(editAccessType, "readaccess", (char *) 0, 0);
  insertRequest (editItem, Req, 0);           /* request definition */
  sprintf(helpText, "Give read access to the current menu.");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  editItem = newItem(editMenu, "change access",EOS);
  setRetPattern(editItem, "$");
  Req = makeOneRequest(editAccessType, "chngaccess", (char *) 0, 0);
  insertRequest (editItem, Req, 0);
  sprintf(helpText, "Give change access to the current menu.");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  /*******************************************************************
   *                   set values of menu MENU_POSITION              *
   ******************************************************************/

  /* create edit menu MENU_POS for Application Node EDIT_MENUS */
  editMenu = newMenu(MENUPOSITION); 
  setSelCnt(editMenu, 2);                     /* set selection count */   
  setSelType(editMenu, MDS_CNT_EQ);           /* set selection type */
  setMenuType(editMenu, MDS_M_PULLD);         /* set menu type */
  setMenuFlag(editMenu, MDS_READACC);         /* set access rights */
  sprintf(helpText, "Options to specify the menu position\non the screen.");
  editMenu->m_hlp->hlpmsg = strCopy(helpText);

  /* create items for edit menu MENU_POS */

  editItem = newItem(editMenu, "position: ", EOS);
  setRetPattern(editItem, "$ $");             /* set return pattern */ 
  Req = makeOneRequest(reqViaWnd, "position (form: '(x, y)'):", 
		       (char *) 0, 10);
  setReqCnt(Req, 2, MDS_CNT_EQ);
  insertRequest (editItem, Req, 0);           /* set request */
  Req = makeOneRequest(editMenuPos1, "0", (char *) 0,
		       0);
  insertRequest (editItem, Req, 1);  
  sprintf(helpText, "Enter coordinates for menu position.");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  editItem = newItem(editMenu, "absolute coordinates",EOS);
  setRetPattern(editItem, "$");
  Req = makeOneRequest(editMenuPos2, "abscoord", (char *) 0, 0);
  insertRequest (editItem, Req, 0);
  sprintf(helpText, "Coordinates for menu position\nare taken as absolute values.");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  setMask(actmask3, 3, 0);                    /* set activity mask */
  actmask3[0] = 1;
  actmask3[1] = 1;
  setItemActivityMask(editMenu, editItem, actmask3); 

  editItem = newItem(editMenu, "relative coordinates", EOS);
  setRetPattern(editItem, "$");
  Req = makeOneRequest(editMenuPos2, "relcoord", (char *) 0, 0);
  insertRequest (editItem, Req, 0);
  sprintf(helpText, "Coordinates for menu position\nare interpreted relative to cursor position.");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  setMask(actmask3, 3, 0);
  actmask3[0] = 1;
  actmask3[2] = 1;
  setItemActivityMask(editMenu, editItem, actmask3);

  /*******************************************************************
   *                   set values of menu ITEM_SELECTION             *
   ******************************************************************/

  /* create edit menu ITEM_SEL for Application Node EDIT_MENUS */
  editMenu = newMenu(ITEMSELECTION); 
  setSelCnt(editMenu,2);                      /* set selection count */
  setSelType(editMenu, MDS_CNT_EQ);           /* set selection type */  
  setMenuType(editMenu, MDS_M_PULLD);         /* set menu type */
  setMenuFlag(editMenu, MDS_READACC);         /* set access rights */
  sprintf(helpText, "Options to specify how many options\ncan be selected in a menu.");
  editMenu->m_hlp->hlpmsg = strCopy(helpText);

  /* create items for edit menu ITEM_SEL */

  editItem = newItem(editMenu, "item selection count", EOS);
  setRetPattern(editItem, "$ $");             /* set return pattern */
  Req = makeOneRequest(reqViaWnd, "number of selectable items:", 
		       (char *) 0, E_ENTRYNUMBER);
  insertRequest (editItem, Req, 0);
  Req = makeOneRequest(editSelCnt, "0", (char *) 0, 0);
  insertRequest (editItem, Req, 1);
  sprintf(helpText, "Specify number of options\n which can be selected in the current menu.");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  editItem = newItem(editMenu, "equal", EOS);
  setRetPattern(editItem, "$");
  Req = makeOneRequest(editSelType, "equal", (char *) 0, 0);
  insertRequest (editItem, Req, 0);
  sprintf(helpText, "Determines that the number of the selected options\nmust be equal to the specified value\nof the option <<item selection count>>.");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  setMask(actmask6, 6, 0);
  actmask6[0] = 1;
  actmask6[1] = 1;
  setItemActivityMask(editMenu, editItem, actmask6);  /*set activity mask */

  editItem = newItem(editMenu, "less",EOS); 
  setRetPattern(editItem, "$");
  Req = makeOneRequest(editSelType, "less", (char *) 0, 0);
  insertRequest (editItem, Req, 0);
  sprintf(helpText, "Determines that the number of the selected options\nmust be less than the specified value\nof the option <<item selection count>>.");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  setMask(actmask6, 6, 0);
  actmask6[0] = 1;
  actmask6[2] = 1;
  setItemActivityMask(editMenu, editItem, actmask6);

  editItem = newItem(editMenu, "less or equal", EOS);
  setRetPattern(editItem, "$");
  Req = makeOneRequest(editSelType, "eqless", (char *) 0, 0);
  insertRequest (editItem, Req, 0);
  sprintf(helpText, "Determines that the number of the selected options\nmust be less than or equal to the specified value\nof the option <<item selection count>>.");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  setMask(actmask6, 6, 0);
  actmask6[0] = 1;
  actmask6[3] = 1;
  setItemActivityMask(editMenu, editItem, actmask6);

  editItem = newItem(editMenu, "greater",EOS);
  setRetPattern(editItem, "$");
  Req = makeOneRequest(editSelType, "greater", (char *) 0, 0);
  insertRequest (editItem, Req, 0);
  sprintf(helpText, "Determines that the number of the selected options\nmust be greater than the specified value\nof the option <<item selection count>>.");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  setMask(actmask6, 6, 0);
  actmask6[0] = 1;
  actmask6[4] = 1;
  setItemActivityMask(editMenu, editItem, actmask6);

  editItem = newItem(editMenu, "greater or equal", EOS);
  setRetPattern(editItem, "$");
  Req = makeOneRequest(editSelType, "eqgreater", (char *) 0, 0);
  insertRequest (editItem, Req, 0);
  sprintf(helpText, "Determines that the number of the selected options\nmust be greater than or equal to the specified value\nof the option <<item selection count>>.");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  setMask(actmask6, 6, 0);
  actmask6[0] = 1;
  actmask6[5] = 1;
  setItemActivityMask(editMenu, editItem, actmask6);

/*  editItem = newItem(editMenu, "default item(s)", EOS); 
    setRetPattern(editItem, "$"); */

  /*******************************************************************
   *                   set values of menu EDIT_FEATURES              *
   ******************************************************************/

  /* create edit menu EDIT_FEATURES for Application Node EDIT_MENUS */
  editMenu = newMenu(EDITFEATURES); 
  setSelCnt(editMenu, 1);                          /* set selection count */
  setSelType(editMenu, MDS_CNT_EQ);                /* set selection type */
  setMenuType(editMenu, MDS_M_PULLD);              /* set menu type */
  setMenuFlag(editMenu, MDS_READACC);              /* set access rights */
  sprintf(helpText, "Special editors to define or manipulate\nfurther features of a menu.");
  editMenu->m_hlp->hlpmsg = strCopy(helpText);

  /* create items for edit menu ITEM_REQ */

  editItem = newItem(editMenu, REQUESTDEF, EOS);
  setRetPattern(editItem, "$");
  Req = makeOneRequest(editRequest, (char *) 0, (char *) 0, 0);
  insertRequest (editItem, Req, 0);
  sprintf(helpText, "Define requests for the current menu.");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  editItem = newItem(editMenu, ACTMASK, EOS);
  setRetPattern(editItem, "$");                /* set return pattern */
  Req = makeOneRequest(editMenuActMask, (char *) 0, (char *) 0, 0);
  insertRequest (editItem, Req, 0);            /* set request */
  sprintf(helpText, "Specify the activity mask\nof the current menu.");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  editItem = newItem(editMenu, HELPTEXT, EOS );
  setRetPattern(editItem, "$");                      /* set return pattern */
  Req = makeOneRequest(editMenuHelp, (char *) 0, (char *) 0, 0);
  insertRequest (editItem, Req, 0);                  /* set request */
  sprintf(helpText, "Edit help texts for the current menu.");
  editItem->i_hlp->hlpmsg = strCopy(helpText);

  /**********************************************************************
   *                    store menus to file                             *
   *********************************************************************/

  setFile(EDIT_MENUS, MEDITORMENUS);
  return(dumpToFile(parent)); 
}

static ApplChange()
{
}

static editMenuType()
{
}

static editMenuPos1()
{
}

static editMenuPos2()
{
}

static editMenuActMask()
{
}

static editSelCnt()
{
}

static editSelType()
{
}

static editRequest()
{
}

static adminSave()
{
}

static adminQuit()
{
}

static adminNewMenu()
{
}

static adminNewSet()
{
}

static adminDuplSet()
{
}

static adminDuplMenu()
{
}

static adminPreview()
{
}

static adminUserFunct()
{
}

static editAccessType()
{
}

static editMenuHelp()
{
}

static newApplList()
{
}

static newMenuList()
{
}

static previewMenu()
{
}



